@@ -1,7 +1,7 @@
 /*
 #   agc.c: a fast lookahead microphone AGC
 #   Copyright (C) 2008 Stefan Fendt      (stefan@sfendt.de)
-#   Copyright (C) 2008-2010 Stephen Fairchild (s-fairchild@users.sourceforge.net)
+#   Copyright (C) 2008-2023 Stephen Fairchild (s-fairchild@users.sourceforge.net)
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 
 #include "agc.h"
 
-/* coefficients of agc_RC_Filter */   
+/* coefficients of agc_RC_Filter */
 struct agc_RC_Coe
     {
     float a;
@@ -52,7 +52,7 @@ struct agc_RC_Filter
     struct agc_RC_Coe coe;
     struct agc_RC_Var var;
     };
-    
+
 struct agc_RC_FilterGroup
     {
     struct agc_RC_Filter RC_HPF_initial[4];
@@ -61,7 +61,7 @@ struct agc_RC_FilterGroup
     struct agc_RC_Filter RC_PHR[4];
     struct agc_RC_Filter RC_F_DS;
     };
-    
+
 struct agc
     {
     int id;
@@ -99,7 +99,7 @@ struct agc
     int hpstages;
     float hf_detail;
     float lf_detail;
-    int use_phaserotator;   
+    int use_phaserotator;
     struct agc_RC_FilterGroup filters;
     };
 
@@ -132,7 +132,7 @@ static float agc_phaserotate(struct agc_RC_Filter *f, float input)
     {
     struct agc_RC_Coe *c = &f->coe;
     struct agc_RC_Var *v = &f->var;
-        
+
     v->hp = c->c * (v->hp + input - v->last_in);
     v->lp = v->lp * c->a + input * c->b;
     v->last_in = input;
@@ -148,14 +148,14 @@ void agc_process_stage1(struct agc *s, float input)
         input = agc_12db_hpfilter(&s->host->filters.RC_HPF_initial[i].coe, &s->filters.RC_HPF_initial[i].var, input);
 
     /* RC-Network (but with only one stage and without resonance/feedback (->6dB/octave))
-     * used as HF-Detail-Filter 
+     * used as HF-Detail-Filter
      */
     input = agc_6db_hpfilter(s->host->hf_detail, &s->host->filters.RC_HPF_detail.coe, &s->filters.RC_HPF_detail.var, input);
 
     /* RC-Network (but with only one stage and without resonance/feedback)
-     * used as LF-Detail-Filter 
+     * used as LF-Detail-Filter
      */
-    input = agc_6db_lpfilter(s->host->lf_detail, &s->host->filters.RC_LPF_detail.coe, &s->filters.RC_LPF_detail.var, input); 
+    input = agc_6db_lpfilter(s->host->lf_detail, &s->host->filters.RC_LPF_detail.coe, &s->filters.RC_LPF_detail.var, input);
 
     /* Phase-rotator done with RC-simulation
      * for good reasons doesn't use Q/resonance either...
@@ -166,18 +166,18 @@ void agc_process_stage1(struct agc *s, float input)
 
     /* feed input into ring-buffer, store input */
     s->buffer[s->in_pos % s->buffer_len] = s->input = input;
-    
-    /* update pointers of the ring-buffer */  
+
+    /* update pointers of the ring-buffer */
     s->in_pos++;
     s->out_pos++;
     }
 
 static float agc_quad_rr(float *storage, int *reset_point, int phase, float input)
     {
-    float highest = 0.0f; 
-    
+    float highest = 0.0f;
+
     input = fabsf(input);
-    
+
     for (int i = 0; i < 4; ++i, ++storage, ++reset_point)
         {
         if (*reset_point == phase)
@@ -208,15 +208,15 @@ void agc_process_stage2(struct agc *s, int mic_is_mute)
     if (s == s->host)
         {
         input = (s->partner->host == s) ? (s->input + s->partner->input) * 0.5 : s->input;
-        phase = s->in_pos % (2 * s->buffer_len);   
-      
+        phase = s->in_pos % (2 * s->buffer_len);
+
         /* De-Esser sidechain-filter - does high and low pass filtering
          */
         {
             float ds_input;
             struct agc_RC_Coe *c = &s->filters.RC_F_DS.coe;
             struct agc_RC_Var *v = &s->filters.RC_F_DS.var;
-            
+
             ds_input = input;
             ds_input += c->q * v->bp;
             v->lp = v->lp * c->a + ds_input * c->b;
@@ -224,11 +224,11 @@ void agc_process_stage2(struct agc *s, int mic_is_mute)
             v->bp = v->bp * c->a + v->hp * c->b;
             v->last_in = ds_input;
         }
-        
+
         /* follow the envelope of the de-esser high and low pass filtered signal */
         ds_amph = agc_quad_rr(s->RR_DS_high, s->RR_reset_point, phase, s->filters.RC_F_DS.var.hp);
         ds_ampl = agc_quad_rr(s->RR_DS_low, s->RR_reset_point, phase, s->filters.RC_F_DS.var.lp);
-        
+
         /* round-robin-4-peak-envelope-follower tracking the general signal level */
         amp = agc_quad_rr(s->RR_signal, s->RR_reset_point, phase, input);
 
@@ -253,13 +253,13 @@ void agc_process_stage2(struct agc *s, int mic_is_mute)
         /* if de-esser says there are only high frequencies, attenuate signal */
         if (ds_amph * s->ds_bias > ds_ampl * 1.3333333f)
              s->ds_state = 1;
- 
+
         if (ds_amph * s->ds_bias < ds_ampl * 0.75f)
              s->ds_state = 0;
- 
+
         if (s->ds_state == 1)
              factor *= s->ds_gain;
- 
+
         /* modulate gain-factor */
         if (s->gain < factor)
             s->gain += s->gain_interval_amount;
@@ -278,17 +278,7 @@ void agc_process_stage2(struct agc *s, int mic_is_mute)
         else
             {
             /* calculate ducking factor */
-            duck_amp = 1.0f - factor * amp;
-        
-            /* if duck-amp is below the minimum-allowed level (limit enforces some headroom)
-             * then limit duck-amp to that minimum-allowed level. This ensures, that if the
-             * microphone-headroom is set to a sensible value (-2..-3dB) there still is some
-             * music audiable in the background...
-             */
-            if (duck_amp < 1.0f - s->limit)
-                {
-                duck_amp = 1.0f - s->limit;
-                }
+            duck_amp = 1.0f - factor * amp / s->limit;
 
             /* ducker is "opened" fast (same rate as agc -> 10ms)
              * but closed more slowly...
@@ -314,7 +304,7 @@ void agc_process_stage2(struct agc *s, int mic_is_mute)
             }
 
         /* maintain a peak hold gain figure for the GUI compression meter
-         * essentially this is metadata 
+         * essentially this is metadata
          */
         if ((s->out_pos & 0x7) == 0)
             {
@@ -337,17 +327,17 @@ void agc_get_meter_levels(struct agc *s, int *signal_cap, int *de_ess, int *nois
         {
         return (int)(log10f(level) * -20.0f);
         }
-        
+
     *signal_cap = (int)level2db(s->meter_signal_cap);
     *de_ess = (int)level2db(s->meter_de_ess);
     *noise_gate = (int)level2db(s->meter_noise_gate);
     }
-    
+
 float agc_get_ducking_factor(struct agc *s)
     {
     return s->df;
     }
-    
+
 void agc_reset(struct agc *s)
     {
     struct agc_RC_Filter *f, *end;
@@ -356,12 +346,12 @@ void agc_reset(struct agc *s)
     for (f = (struct agc_RC_Filter *)&s->filters,
         end = (struct agc_RC_Filter *)((&s->filters) + 1); f < end; ++f)
         {
-        memset(&f->var, 0, sizeof f->var); 
+        memset(&f->var, 0, sizeof f->var);
         }
 
     /* wipe audio buffer */
     memset(s->buffer, 0, s->buffer_len);
- 
+
     /* wipe indicator settings */
     s->df = s->meter_signal_cap = s->meter_de_ess = s->meter_noise_gate = 1.0f;
     }
@@ -375,11 +365,11 @@ static void setup_ratio(struct agc *s, float ratio_db)
 static void setup_subsonic(struct agc *s, float fCutoff)
     {
     struct agc_RC_Coe *c;
-    
+
     for (int i = 0; i < 4; ++i)
         {
         c = &s->filters.RC_HPF_initial[i].coe;
-    
+
         c->f = fCutoff;
         c->q = 0.375f;
         c->a = 1.0f - (1.0f/s->sRate) / ((1.0f / (c->f * 2.0f * M_PI)) + (1.0f/s->sRate));
@@ -391,7 +381,7 @@ static void setup_subsonic(struct agc *s, float fCutoff)
 static void setup_lfdetail(struct agc *s, float multi, float fCutoff)
     {
     struct agc_RC_Coe *c = &s->filters.RC_LPF_detail.coe;
-        
+
     s->lf_detail = multi;
     c->f = fCutoff;
     c->q = 0.375f;
@@ -402,8 +392,8 @@ static void setup_lfdetail(struct agc *s, float multi, float fCutoff)
 
 static void setup_hfdetail(struct agc *s, float multi, float fCutoff)
     {
-    struct agc_RC_Coe *c = &s->filters.RC_HPF_detail.coe;   
-        
+    struct agc_RC_Coe *c = &s->filters.RC_HPF_detail.coe;
+
     s->hf_detail = multi;
     c->f = fCutoff;
     c->q = 0.375f;
@@ -415,13 +405,13 @@ static void setup_hfdetail(struct agc *s, float multi, float fCutoff)
 static void control_phaserotate(struct agc *s, char *value)
     {
     s->use_phaserotator = (value[0] == '1');
-    } 
+    }
 
 static void control_gain(struct agc *s, char *value)
     {
     setup_ratio(s, strtof(value, NULL));
     }
-    
+
 static void control_limit(struct agc *s, char *value)
     {
     s->limit = powf(2.0f, strtof(value, NULL) / 6.0f);
@@ -452,7 +442,7 @@ static void control_duckhold(struct agc *s, char *value)
     {
     s->ducker_hold_timer_resetval = atoi(value) * s->sRate / 1000;
     }
-    
+
 static void control_deessbias(struct agc *s, char *value)
     {
     s->ds_bias = strtof(value, NULL);
@@ -509,7 +499,7 @@ static void setup_control_hash_table()
         {"gain", control_gain},
         {"limit", control_limit},
         {"ngthresh", control_ngthresh},
-        {"nggain", control_nggain}, 
+        {"nggain", control_nggain},
         {"duckenable", control_duckenable},
         {"duckrelease", control_duckrelease},
         {"duckhold", control_duckhold},
@@ -528,17 +518,17 @@ static void setup_control_hash_table()
         fprintf(stderr, "agc setup_control_hash_table failed\n");
         return;
         }
-    
+
     for (htdp = htdata; htdp->key; ++htdp)
         g_hash_table_insert(control_ht, htdp->key, htdp->value);
-        
+
     atexit(free_control_hash_table);
     }
 
 void agc_control(struct agc *s, char *key, char *value)
     {
     void (*fn)(struct agc *, char *);
-    
+
     if (!(fn = g_hash_table_lookup(control_ht, key)))
         fprintf(stderr, "agc_control: lookup error for key %s\n", key);
     else
@@ -549,7 +539,7 @@ void agc_set_as_partners(struct agc *agc1, struct agc *agc2)
     {
     agc1->partner = agc2;
     agc2->partner = agc1;
-    } 
+    }
 
 void agc_set_partnered_mode(struct agc *s, int boolean)
     {
@@ -564,7 +554,7 @@ struct agc *agc_init(int sRate, float lookahead, int id)
     struct agc *s;
     struct agc_RC_Coe *c;
     static pthread_once_t control_hash_table_once = PTHREAD_ONCE_INIT;
-    
+
     pthread_once(&control_hash_table_once, setup_control_hash_table);
 
     if (!(s = calloc(1, sizeof (struct agc))))
@@ -601,40 +591,40 @@ struct agc *agc_init(int sRate, float lookahead, int id)
     s->nr_onthres = 0.1f;      /* silence detection level */
     s->nr_offthres = 0.1001f;  /* non-silence detection level */
     s->nr_gain = 0.5f;         /* if silence detected reduce gain by 6dB */
-    
+
     s->ds_bias  =  0.35f; /* lpf * bias / hpf exceeds 1 for de-esser to go active */
     s->ds_gain  =  0.5f;  /* attenuate signal by this amount */
     s->meter_signal_cap = s->meter_de_ess = s->meter_noise_gate = 1.0f;
-    
+
     /* setup coefficients for the ducker */
     s->ducker_release = 1.0f / (0.250f * s->sRate); /* 250ms */
     s->ducker_attack  = 1.0f / s->buffer_len;    /* same as lookahead delay */
     s->ducker_hold_timer_resetval = 0.500f * s->sRate; /* 500ms */
     s->df = 1.0f;
-    
+
     /* setup coefficients for the subsonic-and-DC-killer-RC-highpass */
     setup_subsonic(s, 100.0f);
     s->hpstages = 4;
-    
+
     /* setup coefficients for the HF-Detail highpass */
     setup_hfdetail(s, 4.0f, 2000.0f);
-    
+
     /* setup coefficients for the LF-Detail lowpass */
     setup_lfdetail(s, 4.0f, 150.0f);
-    
+
     /* setup coefficients for the phase rotator */
     s->use_phaserotator = 1;
     for (int i = 0; i < 4; ++i)
         {
         c = &s->filters.RC_PHR[i].coe;
-        
+
         c->f = 300.0f;
         c->q = 0.0f;
         c->a = 1.0f - (1.0f / s->sRate) / ((1.0f/(c->f * 2.0f * M_PI)) + (1.0f / s->sRate));
         c->b = 1.0f - c->a;
         c->c = (1.0f / (c->f * 2.0f * M_PI)) / ((1.0f / (c->f * 2.0f * M_PI)) + (1.0f / s->sRate));
         }
-    
+
     /* setup coefficients for the de-esser-sidechain highpass/lowpass filter */
     c = &s->filters.RC_F_DS.coe;
     c->f = 1000.0f;
