@@ -34,63 +34,50 @@ int audio_feed_process_audio(jack_nframes_t n_frames, void *arg)
     {
     struct audio_feed *self = audio_feed;
     struct threads_info *ti = self->threads_info;
-    struct encoder *e;
-    struct recorder *r;
-    sample_t *input_port_buffer[2];
+    sample_t *input_port_buffer[2], *monitor_buffer[2];
     int i;
-    
+
     input_port_buffer[0] = jack_port_get_buffer(g.port.output_in_l, n_frames);
     input_port_buffer[1] = jack_port_get_buffer(g.port.output_in_r, n_frames);
-    
-    /* feed pcm audio data to all encoders that request it */
-    for (i = 0; i < ti->n_encoders; i++)
+    monitor_buffer[0] = jack_port_get_buffer(g.port.monitor_output_in_l, n_frames);
+    monitor_buffer[1] = jack_port_get_buffer(g.port.monitor_output_in_r, n_frames);
+
+    for (int i = 0; i < 2; ++i)
+        memcpy(monitor_buffer[i], input_port_buffer[i], n_frames * sizeof (sample_t));
+
+    void process(struct audio_feed_data *afdata)
         {
-        e = ti->encoder[i];
-        switch (e->jack_dataflow_control)
+        reevaluate:
+        switch (afdata->jack_dataflow_control)
             {
             case JD_OFF:
                 break;
             case JD_ON:
-                while (jack_ringbuffer_write_space(e->input_rb[1]) < n_frames * sizeof (sample_t))
-                    nanosleep(&(struct timespec){0, 10000000}, NULL);
-                    
-                jack_ringbuffer_write(e->input_rb[0], (char *)input_port_buffer[0], n_frames * sizeof (sample_t));
-                jack_ringbuffer_write(e->input_rb[1], (char *)input_port_buffer[1], n_frames * sizeof (sample_t));
+                while (jack_ringbuffer_write_space(afdata->input_rb[1]) < n_frames * sizeof (sample_t)) {
+                    nanosleep(&(struct timespec){0, 100000}, NULL);
+                    if (afdata->jack_dataflow_control != JD_ON)
+                        goto reevaluate;
+                }
+
+                jack_ringbuffer_write(afdata->input_rb[0], (char *)input_port_buffer[0], n_frames * sizeof (sample_t));
+                jack_ringbuffer_write(afdata->input_rb[1], (char *)input_port_buffer[1], n_frames * sizeof (sample_t));
                 break;
             case JD_FLUSH:
-                jack_ringbuffer_reset(e->input_rb[0]);
-                jack_ringbuffer_reset(e->input_rb[1]);
-                e->jack_dataflow_control = JD_OFF;
+                jack_ringbuffer_reset(afdata->input_rb[0]);
+                jack_ringbuffer_reset(afdata->input_rb[1]);
+                afdata->jack_dataflow_control = JD_OFF;
                 break;
             default:
                 fprintf(stderr, "jack_process_callback: unhandled jack_dataflow_control parameter\n");
             }
         }
-        
-    for (i = 0; i < ti->n_recorders; i++)
-        {
-        r = ti->recorder[i];
-        switch (r->jack_dataflow_control)
-            {
-            case JD_OFF:
-                break;
-            case JD_ON:
-                while (jack_ringbuffer_write_space(r->input_rb[1]) < n_frames * sizeof (sample_t))
-                    nanosleep(&(struct timespec){0, 10000000}, NULL);                
 
-                jack_ringbuffer_write(r->input_rb[0], (char *)input_port_buffer[0], n_frames * sizeof (sample_t));
-                jack_ringbuffer_write(r->input_rb[1], (char *)input_port_buffer[1], n_frames * sizeof (sample_t));
-                break;
-            case JD_FLUSH:
-                jack_ringbuffer_reset(r->input_rb[0]);
-                jack_ringbuffer_reset(r->input_rb[1]);
-                r->jack_dataflow_control = JD_OFF;
-                break;
-            default:
-                fprintf(stderr, "jack_process_callback: unhandled jack_dataflow_control parameter\n");
-            }   
-        }
-      
+    for (i = 0; i < ti->n_encoders; i++)
+        process(&(ti->encoder[i]->afdata));
+
+    for (i = 0; i < ti->n_recorders; i++)
+        process(&(ti->recorder[i]->afdata));
+
     return 0;
     }
 
@@ -113,7 +100,7 @@ struct audio_feed *audio_feed_init(struct threads_info *ti)
         return NULL;
         }
 
-    self->threads_info = ti;      
+    self->threads_info = ti;
     self->sample_rate = jack_get_sample_rate(g.client);
     return self;
     }
