@@ -1,6 +1,6 @@
 /*
-#   avcodecdecode.c: decodes wma file format for xlplayer
-#   Copyright (C) 2007-2017 Stephen Fairchild (s-fairchild@users.sf.net)
+#   avcodecdecode.c: decodes wma, mp4, and other file formats for xlplayer
+#   Copyright (C) 2007-2022 Stephen Fairchild (s-fairchild@users.sf.net)
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -25,9 +25,7 @@
 #include <stdlib.h>
 #include <pthread.h>
 #include <libavutil/opt.h>
-#ifdef HAVE_LIBAVUTIL_CHANNEL_LAYOUT_H
 #include <libavutil/channel_layout.h>
-#endif
 #include <libavutil/samplefmt.h>
 #include "main.h"
 #include "xlplayer.h"
@@ -42,32 +40,26 @@
 #define AVCODEC_MAX_AUDIO_FRAME_SIZE 192000
 #endif
 
-#ifndef HAVE_AV_FRAME_ALLOC
-#define av_frame_alloc avcodec_alloc_frame
-#endif
-#ifndef HAVE_AV_FRAME_UNREF
-#define av_frame_unref avcodec_get_frame_defaults
-#endif
-#ifndef HAVE_AV_PACKET_UNREF
-#define av_packet_unref av_free_packet
-#endif
 
 extern int dynamic_metadata_form[];
 
 static const struct timespec time_delay = { .tv_nsec = 10 };
 
+
 static void write_frame(struct xlplayer *xlplayer, AVFrame *frame)
     {
     struct avcodecdecode_vars *self = xlplayer->dec_data;
     int frames;
-    int channels = self->c->channels;
+    int channels = self->c->ch_layout.nb_channels;
     SRC_DATA *src_data = &xlplayer->src_data;
 
-#if defined (HAVE_SWRESAMPLE) && defined(USE_SWRESAMPLE)
+    if (channels == 0) {
+        fprintf(stderr, "avcodecdecode_play: number of channels is 0\n");
+        xlplayer->playmode = PM_EJECTING;
+    }
+
     if (!self->swr)
         {
-        int64_t layout;
-
         if (!(self->swr = swr_alloc()))
             {
             fprintf(stderr, "avcodecdecode_play: call to swr_alloc failed\n");
@@ -75,25 +67,14 @@ static void write_frame(struct xlplayer *xlplayer, AVFrame *frame)
             return;
             }
 
-        layout = frame->channel_layout;
-        if (!layout)
-            layout = self->c->channel_layout;
-        if (!layout)
-            {
-            if (!channels)
-                {
-                fprintf(stderr, "avcodecdecode_play: number of channels is zero\n");
-                xlplayer->playmode = PM_EJECTING;
-                return;
-                }
-
-            layout = av_get_default_channel_layout(channels);
-            }
-
-        av_opt_set_int(self->swr, "in_channel_layout", layout, 0);
-        av_opt_set_int(self->swr, "out_channel_layout", (self->channels == 2) ? AV_CH_LAYOUT_STEREO : AV_CH_LAYOUT_MONO, 0);
+        av_opt_set_chlayout(self->swr, "in_chlayout", &self->c->ch_layout, 0);
+        av_opt_set_chlayout(self->swr, "out_chlayout", &self->out_ch_layout, 0);
         av_opt_set_sample_fmt(self->swr, "in_sample_fmt", self->c->sample_fmt, 0);
         av_opt_set_sample_fmt(self->swr, "out_sample_fmt", AV_SAMPLE_FMT_FLT, 0);
+        av_opt_set_int(self->swr, "in_sample_rate", self->c->sample_rate, 0);
+        /* We are using libswresample only for the channel layout conversion
+         * therefore the output sample rate must match the input. */
+        av_opt_set_int(self->swr, "out_sample_rate", self->c->sample_rate, 0);
 
         if (swr_init(self->swr))
             {
@@ -114,141 +95,6 @@ static void write_frame(struct xlplayer *xlplayer, AVFrame *frame)
         }
 
     swr_convert(self->swr, &self->floatsamples, frame->nb_samples, (const uint8_t **)frame->data, frame->nb_samples);
-#else
-
-    if (!(self->floatsamples))
-        {
-        if (channels > 2 || channels < 1)
-            {
-            fprintf(stderr, "avcodecdecode_init: unhandled number of channels: %d\n", channels);
-            xlplayer->playmode = PM_EJECTING;
-            return;
-            }
-
-        if (!(self->floatsamples = malloc(sizeof (float) * self->channels * AVCODEC_MAX_AUDIO_FRAME_SIZE)))
-            {
-            fprintf(stderr, "avcodecdecode_init: malloc failure\n");
-            xlplayer->playmode = PM_EJECTING;
-            return;
-            }
-        }
-
-    int buffer_size = av_samples_get_buffer_size(NULL, channels,
-                        frame->nb_samples, self->c->sample_fmt, 1);
-
-    switch (self->c->sample_fmt) {
-        case AV_SAMPLE_FMT_FLT:
-            frames = (buffer_size >> 2) / channels;
-            memcpy(self->floatsamples, frame->data[0], buffer_size);
-            break;
-
-        case AV_SAMPLE_FMT_FLTP:
-            frames = (buffer_size >> 2) / channels;
-            {
-            float *l = (float *)frame->data[0];
-            float *r = NULL;
-            if (channels == 2)
-                r = (float *)frame->data[1];
-            float *d = self->floatsamples;
-            float *endp = self->floatsamples + (channels * frames);
-            while (d < endp)
-                {
-                *d++ = *l++;
-                if (channels == 2)
-                    *d++ = *r++;
-                }
-            }
-            break;
-
-        case AV_SAMPLE_FMT_DBL:
-            frames  = (buffer_size >> 3) / channels;
-            {
-            double *s = (double *)frame->data[0];
-            float *d = self->floatsamples;
-            float *endp = self->floatsamples + (channels * frames);
-            while (d < endp)
-                *d++ = (float)*s++;
-            }
-            break;
-
-        case AV_SAMPLE_FMT_DBLP:
-            frames  = (buffer_size >> 3) / channels;
-            {
-            double *l = (double *)frame->data[0];
-            double *r = NULL;
-            if (channels == 2)
-                r = (double *)frame->data[1];
-            float *d = self->floatsamples;
-            float *endp = self->floatsamples + (channels * frames);
-            while (d < endp)
-                {
-                *d++ = (float)*l++;
-                if (channels == 2)
-                    *d++ = (float)*r++;
-                }
-            }
-            break;
-
-        case AV_SAMPLE_FMT_S16:
-             frames = (buffer_size >> 1) / channels;
-            xlplayer_make_audio_to_float(xlplayer, self->floatsamples,
-                            frame->data[0], frames, 16, channels);
-            break;
-
-        case AV_SAMPLE_FMT_S16P:
-            frames = (buffer_size >> 1) / channels;
-            {
-            int16_t *l = (int16_t *)frame->data[0];
-            int16_t *r = NULL;
-            if (channels == 2)
-                r = (int16_t *)frame->data[1];
-            float *d = self->floatsamples;
-            float *endp = self->floatsamples + (channels * frames);
-            while (d < endp)
-                {
-                *d++ = *l++ / 32768.0f;
-                if (channels == 2)
-                    *d++ = *r++ / 32768.0f;
-                }
-            }
-            break;
-
-        case AV_SAMPLE_FMT_S32:
-            frames = (buffer_size >> 2) / channels;
-            xlplayer_make_audio_to_float(xlplayer, self->floatsamples,
-                            frame->data[0], frames, 32, channels);
-            break;
-
-        case AV_SAMPLE_FMT_S32P:
-            frames = (buffer_size >> 2) / channels;
-            {
-            int32_t *l = (int32_t *)frame->data[0];
-            int32_t *r = NULL;
-            if (channels == 2)
-                r = (int32_t *)frame->data[1];
-            float *d = self->floatsamples;
-            float *endp = self->floatsamples + (channels * frames);
-            while (d < endp)
-                {
-                *d++ = *l++ / 1073741824.0f;
-                if (channels == 2)
-                    *d++ = *r++ / 1073741824.0f;
-                }
-            }
-            break;
-
-        case AV_SAMPLE_FMT_NONE:
-            fprintf(stderr, "avcodecdecode_play: sample format is none\n");
-            xlplayer->playmode = PM_EJECTING;
-            return;
-
-        default:
-            fprintf(stderr, "avcodecdecode_play: unexpected data format %d\n", (int)self->c->sample_fmt);
-            xlplayer->playmode = PM_EJECTING;
-            return;
-        }
-
-#endif /* defined (HAVE_SWRESAMPLE) && defined(USE_SWRESAMPLE) */
 
     if (self->resample)
         {
@@ -286,19 +132,14 @@ static void avcodecdecode_eject(struct xlplayer *xlplayer)
         }
     if (self->floatsamples)
         free(self->floatsamples);
-#ifdef HAVE_SWRESAMPLE
     if (self->swr)
         swr_free(&self->swr);
-#endif
     while (pthread_mutex_trylock(&g.avc_mutex))
         nanosleep(&time_delay, NULL);
     avcodec_free_context(&self->c);
     pthread_mutex_unlock(&g.avc_mutex);
     avformat_close_input(&self->ic);
-#ifndef HAVE_AVCODEC_SEND_PACKET
-    if (self->frame)
-        av_freep(&self->frame);
-#endif
+    av_channel_layout_uninit(&self->out_ch_layout);
     free(self);
     fprintf(stderr, "finished eject\n");
     }
@@ -322,7 +163,20 @@ static void avcodecdecode_init(struct xlplayer *xlplayer)
             }
         }
 
-    self->channels = (self->c->channels == 1) ? 1 : 2;
+    switch(self->c->ch_layout.nb_channels) {
+        case 2:
+            av_channel_layout_from_string(&self->out_ch_layout, "stereo");
+            self->channels = 2;
+            break;
+        case 1:
+            av_channel_layout_from_string(&self->out_ch_layout, "mono");
+            self->channels = 1;
+            break;
+        default:
+            av_channel_layout_from_string(&self->out_ch_layout, "downmix");
+            self->channels = 2;
+    }
+
     if ((self->resample = (self->c->sample_rate != (int)xlplayer->samplerate)))
         {
         fprintf(stderr, "configuring resampler\n");
@@ -356,12 +210,11 @@ static void avcodecdecode_init(struct xlplayer *xlplayer)
     fprintf(stderr, "avcodecdecode_init: completed\n");
     }
 
-#ifdef HAVE_AVCODEC_SEND_PACKET
 
 static void avcodecdecode_play(struct xlplayer *xlplayer)
 {
     struct avcodecdecode_vars *self = xlplayer->dec_data;
-    int channels = self->c->channels;
+    int channels = self->c->ch_layout.nb_channels;
     SRC_DATA *src_data = &xlplayer->src_data;
 
     for(;;) {
@@ -419,8 +272,7 @@ static void avcodecdecode_play(struct xlplayer *xlplayer)
     return;
 
 cleanup:
-    if (self->af.data)
-        av_frame_unref(&self->af);
+    av_frame_unref(&self->af);
 
     if (self->pkt.data)
         av_packet_unref(&self->pkt);
@@ -441,103 +293,6 @@ cleanup:
     xlplayer->playmode = PM_FLUSH;
 }
 
-#else
-
-static void avcodecdecode_play(struct xlplayer *xlplayer)
-    {
-    struct avcodecdecode_vars *self = xlplayer->dec_data;
-    int channels = self->c->channels;
-    SRC_DATA *src_data = &xlplayer->src_data;
-
-    if (self->size <= 0)
-        {
-        if (av_read_frame(self->ic, &self->pkt) < 0 || (self->size = self->pkt.size) == 0)
-            {
-            if (self->pkt.data)
-                av_packet_unref(&self->pkt);
-
-            if (self->resample)       /* flush the resampler */
-                {
-                src_data->end_of_input = TRUE;
-                src_data->input_frames = 0;
-                if (src_process(xlplayer->src_state, src_data))
-                    {
-                    fprintf(stderr, "avcodecdecode_play: error occured during resampling\n");
-                    xlplayer->playmode = PM_EJECTING;
-                    return;
-                    }
-                xlplayer_demux_channel_data(xlplayer, src_data->data_out, src_data->output_frames_gen, channels, 1.f);
-                xlplayer_write_channel_data(xlplayer);
-                }
-            xlplayer->playmode = PM_FLUSH;
-            return;
-            }
-        self->pktcopy = self->pkt;
-        }
-
-    if (self->pkt.stream_index != (int)self->stream)
-        {
-        if (self->pkt.data)
-            av_packet_unref(&self->pkt);
-        self->size = 0;
-        return;
-        }
-
-    do
-        {
-        int len, frames, got_frame = 0;
-
-        if (!self->frame)
-            {
-            if (!(self->frame = av_frame_alloc()))
-                {
-                fprintf(stderr, "avcodecdecode_play: malloc failure\n");
-                exit(1);
-                }
-            else
-                av_frame_unref(self->frame);
-            }
-
-        while (pthread_mutex_trylock(&g.avc_mutex))
-            nanosleep(&time_delay, NULL);
-        len = avcodec_decode_audio4(self->c, self->frame, &got_frame, &self->pktcopy);
-        pthread_mutex_unlock(&g.avc_mutex);
-
-        if (len < 0)
-            {
-            fprintf(stderr, "avcodecdecode_play: error during decode\n");
-            break;
-            }
-
-        self->pktcopy.data += len;
-        self->pktcopy.size -= len;
-        self->size -= len;
-
-        if (!got_frame)
-            {
-            continue;
-            }
-
-        write_frame(xlplayer, self->frame);
-
-        } while (!xlplayer->write_deferred && self->size > 0);
-
-    if (self->size <= 0)
-        {
-        if (self->pkt.data)
-            av_packet_unref(&self->pkt);
-        int delay = xlplayer_calc_rbdelay(xlplayer);
-        struct chapter *chapter = mp3_tag_chapter_scan(&self->taginfo, xlplayer->play_progress_ms + delay);
-        if (chapter && chapter != self->current_chapter)
-            {
-            self->current_chapter = chapter;
-            xlplayer_set_dynamic_metadata(xlplayer, dynamic_metadata_form[chapter->title.encoding], chapter->artist.text, chapter->title.text, chapter->album.text, delay);
-            }
-        }
-    }
-
-#endif /* HAVE_AVCODEC_SEND_PACKET */
-
 
 int avcodecdecode_reg(struct xlplayer *xlplayer)
     {
@@ -585,7 +340,9 @@ int avcodecdecode_reg(struct xlplayer *xlplayer)
 
     while (pthread_mutex_trylock(&g.avc_mutex))
         nanosleep(&time_delay, NULL);
+
     self->stream = av_find_best_stream(self->ic, AVMEDIA_TYPE_AUDIO, -1, -1, &self->codec, 0);
+
     pthread_mutex_unlock(&g.avc_mutex);
 
     if (self->stream < 0)
@@ -613,11 +370,6 @@ int avcodecdecode_reg(struct xlplayer *xlplayer)
 
     }
 
-#ifndef USE_SWRESAMPLE
-    self->c->request_sample_fmt = AV_SAMPLE_FMT_FLT;
-    self->c->request_channel_layout = AV_CH_LAYOUT_STEREO_DOWNMIX;
-#endif
-
     while (pthread_mutex_trylock(&g.avc_mutex))
         nanosleep(&time_delay, NULL);
     if (avcodec_open2(self->c, self->codec, NULL) < 0)
