@@ -1,6 +1,6 @@
 /*
 #   ialloc.c: Heap memory allocation routines for IDJC.
-#   Copyright (C) 2005-2006 Stephen Fairchild (s-fairchild@users.sourceforge.net)
+#   Copyright (C) 2005-2012 Stephen Fairchild (s-fairchild@users.sourceforge.net)
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -21,34 +21,36 @@
 #include <unistd.h>
 #include <jack/jack.h>
 #include <stdio.h>
+#include <assert.h>
 
 typedef jack_default_audio_sample_t sample_t;
 
 sample_t *ialloc(jack_nframes_t size)
-   {
-   jack_nframes_t *memblock;
-   
-   memblock = malloc(size * sizeof (sample_t) + sizeof (jack_nframes_t));
-   *memblock++ = size;
-   return (sample_t *)memblock;
-   }
-  
-void ifree(sample_t *memblock)
-   {
-   free(((jack_nframes_t *)memblock)-1);
-   }
+    {
+    sample_t *buf;
+    
+    if (!(buf = malloc(sizeof (sample_t) * size)))
+        {
+        fprintf(stderr, "ialloc: malloc failure\n");
+        exit(5);
+        }
 
-sample_t *irealloc(sample_t *orig, jack_nframes_t newsize)
-   {
-   jack_nframes_t *oldbuf;
-   
-   oldbuf = ((jack_nframes_t *)orig)-1;
-   if (newsize > *oldbuf)
-      {
-      fprintf(stderr, "Reallocating in irealloc\n");
-      free(oldbuf);
-      return ialloc(newsize);
-      }
-   else
-      return orig;
-   }
+    return buf;
+    }
+  
+void ifree(sample_t *buf)
+    {
+    if (buf)
+        free(buf);
+    }
+
+sample_t *irealloc(sample_t *data, jack_nframes_t newsize)
+    {
+    if (!data)    
+        return ialloc(newsize);
+    else
+        {
+        free(data);
+        return ialloc(newsize);
+        }
+    }
