@@ -1,6 +1,6 @@
 /*
 #   kvpdict.h: key-value pair header file for kvpdict.c, part of the IDJC project
-#   Copyright (C) 2005-2007 Stephen Fairchild (s-fairchild@users.sourceforge.net)
+#   Copyright (C) 2005-2007, 2024 Stephen Fairchild (s-fairchild@users.sourceforge.net)
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -21,6 +21,8 @@
 
 #include <pthread.h>
 
+#include <glib.h>
+
 struct kvpdict
     {
     char *key;           /* the key from a key value pair to match against */
@@ -28,9 +30,9 @@ struct kvpdict
                                 rather than to make the new value a member of the dictionary */
     pthread_mutex_t *pm; /* if a lock is supplied here it will be used */
     };
-    
+
 char *kvp_extract_value(char *keyvaluepair);
-int kvp_apply_to_dict(struct kvpdict *kvpdict, char *key, char *newtarget);
+int kvp_apply_to_ht(GHashTable *ht, char *key, char *newtarget);
 void kvp_free_dict(struct kvpdict *dp);
 
 #endif
