@@ -32,7 +32,7 @@ static void kvp_cleanup()
         free(buffer);
     }
 
-int kvp_parse(struct kvpdict *kvpdict, FILE *fp)
+int kvp_parse(GHashTable *ht, FILE *fp)
     {
     static size_t n = 5000;
     char *value;
@@ -46,14 +46,14 @@ int kvp_parse(struct kvpdict *kvpdict, FILE *fp)
             exit(5);
             }
         atexit(kvp_cleanup);
-        } 
+        }
 
     while (rv = getline(&buffer, &n, fp), rv > 0 && strcmp(buffer, "end\n"))
         {
         /* the following function is fed a key value pair e.g. key=value */
         value = kvp_extract_value(buffer); /* key is truncated at the = */
         /* value = a pointer to a copy of the value part after the '=' allocated on the heap */
-        if(!(kvp_apply_to_dict(kvpdict, buffer, value)))
+        if(!(kvp_apply_to_ht(ht, buffer, value)))
             fprintf(stderr, "kvp_parse: %s=%s, key missing from dictionary\n", buffer, value);
         /* assuming the error message wasn't printed the associated pointer in the dictionary will have been updated */
         }
@@ -62,3 +62,18 @@ int kvp_parse(struct kvpdict *kvpdict, FILE *fp)
         fprintf(stderr, "getline failed to allocate a buffer in function kvp_parse\n");
     return rv > 0;
     }
+
+GHashTable *kvp_ht_init(struct kvpdict *kp)
+{
+    GHashTable *ht = g_hash_table_new(g_str_hash, g_str_equal);
+    if (!ht) {
+        fprintf(stderr, "kvp_ht_init: failed to allocate hash table\n");
+        exit(5);
+    }
+
+    for (; kp->key; kp++) {
+        if (!g_hash_table_insert(ht, kp->key, kp))
+            printf("!!duplicate key %s\n", kp->key);
+    }
+    return ht;
+}
