@@ -19,4 +19,5 @@
 
 #include "kvpdict.h"
 
-int kvp_parse(struct kvpdict *kvpdict, FILE *fp);
+int kvp_parse(GHashTable *ht, FILE *fp);
+GHashTable *kvp_ht_init(struct kvpdict *kp);
