@@ -40,6 +40,14 @@
 #include "sourceclient.h"
 #include "live_aac_encoder.h"
 
+#ifndef AV_PROFILE_AAC_LOW /// ok to remove in year 2027
+#define AV_PROFILE_AAC_LOW FF_PROFILE_AAC_LOW
+#endif
+
+#ifndef AV_PROFILE_AAC_HE_V2 /// ok to remove in year 2027
+#define AV_PROFILE_AAC_HE_V2 FF_PROFILE_AAC_HE_V2
+#endif
+
 static const struct timespec time_delay = { .tv_nsec = 10 };
 
 typedef struct State {
@@ -100,6 +108,26 @@ static const AVCodec *add_stream(State *self, enum AVCodecID codec_id,
         return NULL;
     }
 
+#ifdef HAVE_AVCODEC_GET_SUPPORTED_CONFIG
+    {
+        const void *sfs;
+        int nsfs;
+        enum AVSampleFormat sf;
+        if (!avcodec_get_supported_config(c, codec,
+                                          AV_CODEC_CONFIG_SAMPLE_FORMAT, 0,
+                                          &sfs, &nsfs)) {
+
+
+            for (int i = 0; i < nsfs; ++i) {
+                sf = ((enum AVSampleFormat *)sfs)[i];
+                if (sf == AV_SAMPLE_FMT_FLTP || sf == AV_SAMPLE_FMT_FLT) {
+                    c->sample_fmt = sf;
+                    break;
+                }
+            }
+        }
+    }
+#else
     if (codec->sample_fmts) {
         c->sample_fmt = codec->sample_fmts[0];
         for (int i = 0; codec->sample_fmts[i]; i++) {
@@ -112,8 +140,10 @@ static const AVCodec *add_stream(State *self, enum AVCodecID codec_id,
                 break;
             }
         }
-    } else
+    } else {
         c->sample_fmt = AV_SAMPLE_FMT_FLTP;
+    }
+#endif /* HAVE_AVCODEC_GET_SUPPORTED_CONFIG */
 
     c->bit_rate    = br;
     c->sample_rate = sr;
@@ -444,12 +474,12 @@ static int setup(struct encoder *encoder)
         case ENCODER_CODEC_AAC:
             self->const_packet_flags = PF_AAC;
             codec_id = AV_CODEC_ID_AAC;
-            profile = FF_PROFILE_AAC_LOW;
+            profile = AV_PROFILE_AAC_LOW;
             break;
         case ENCODER_CODEC_AACPLUSV2:
             self->const_packet_flags = PF_AACP2;
             codec_id = AV_CODEC_ID_AAC;
-            profile = FF_PROFILE_AAC_HE_V2;
+            profile = AV_PROFILE_AAC_HE_V2;
             break;
         default:
             goto fail1;
