@@ -75,7 +75,7 @@ void live_ogg_free_metadata(struct ogg_tag_data *t)
 
 int live_ogg_write_packet(struct encoder *encoder, ogg_page *op, int flags)
     {
-    struct encoder_op_packet packet;
+    struct encoder_op_packet_rb_feed packet;
     char *buffer;
 
     if (!(buffer = malloc(op->header_len + op->body_len)))
@@ -109,7 +109,7 @@ static void live_ogg_encoder_main(struct encoder *encoder)
     float **buffer;
     ogg_int64_t oldgranulepos;
     struct ogg_tag_data *t = &s->tag_data;
-    
+
     if (encoder->encoder_state == ES_STARTING)
         {
         fprintf(stderr, "live_ogg_encoder_main: first pass of the encoder\n");
@@ -125,7 +125,7 @@ static void live_ogg_encoder_main(struct encoder *encoder)
         ai.bitrate_limit_min_kbps = s->min_bitrate / 1000;
         if (vorbis_encode_ctl(&s->vi, OV_ECTL_RATEMANAGE2_SET, &ai))
             fprintf(stderr, "live_ogg_encoder_main: failed to set hard bitrate floor\n");
-            
+
         vorbis_encode_setup_init(&s->vi);
         vorbis_analysis_init(&s->vd, &s->vi);
         vorbis_block_init(&s->vd, &s->vb);
@@ -135,7 +135,7 @@ static void live_ogg_encoder_main(struct encoder *encoder)
         /* this function takes raw metadata and does something type specific with it */
         if (encoder->new_metadata)
             live_ogg_capture_metadata(encoder, t);
-        
+
         if (t->custom && t->custom[0])
             {
             vorbis_comment_add_tag(&s->vc, "TITLE", t->custom);
@@ -278,7 +278,7 @@ int live_ogg_encoder_init(struct encoder *encoder, struct encoder_vars *ev)
     else
         {
         long var = encoder->bitrate * atol(ev->variability) / 100;
-        
+
         s->max_bitrate = encoder->bitrate + var;
         s->min_bitrate = encoder->bitrate - var;
         }
