@@ -98,7 +98,7 @@ static FLAC__StreamEncoderWriteStatus live_oggflac_encoder_write_cb(const FLAC__
     {
     struct encoder *encoder = client_data;
     struct lofe_data *s = encoder->encoder_private;
-    struct encoder_op_packet packet;
+    struct encoder_op_packet_rb_feed packet;
     ogg_page og;
     int granulepos;
 
@@ -106,22 +106,22 @@ static FLAC__StreamEncoderWriteStatus live_oggflac_encoder_write_cb(const FLAC__
         {
         /* writing ogg header */
         s->pab_rqd = s->pab_head_size = bytes;
-        
+
         if (s->pab_size < s->pab_rqd)
             if (!(s->pab = realloc(s->pab, s->pab_size = s->pab_rqd)))
                 {
                 fprintf(stderr, "live_oggflac_encoder_write_cb: malloc failure\n");
                 return FLAC__STREAM_ENCODER_WRITE_STATUS_FATAL_ERROR;
                 }
-        
+
         memcpy(s->pab, buffer, bytes);
-        
+
         s->flags = PF_OGG;
         if (s->n_writes == 0)
             s->flags |= PF_INITIAL;
         if (buffer[5] & 0x4)
             s->flags |= PF_FINAL;
-            
+
         og.header = (unsigned char *)buffer;
         og.header_len = bytes;
         og.body = NULL;
@@ -146,9 +146,9 @@ static FLAC__StreamEncoderWriteStatus live_oggflac_encoder_write_cb(const FLAC__
                 fprintf(stderr, "live_oggflac_encoder_write_cb: malloc failure\n");
                 return FLAC__STREAM_ENCODER_WRITE_STATUS_FATAL_ERROR;
                 }
-        
+
         memcpy(s->pab + s->pab_head_size, buffer, bytes);
-        
+
         packet.header.bit_rate = encoder->bitrate;
         packet.header.sample_rate = encoder->target_samplerate;
         packet.header.n_channels = encoder->n_channels;
@@ -158,7 +158,7 @@ static FLAC__StreamEncoderWriteStatus live_oggflac_encoder_write_cb(const FLAC__
         packet.data = s->pab;
         encoder_write_packet_all(encoder, &packet);
         }
-        
+
     s->n_writes++;
     return FLAC__STREAM_ENCODER_WRITE_STATUS_OK;
     }
@@ -166,13 +166,13 @@ static FLAC__StreamEncoderWriteStatus live_oggflac_encoder_write_cb(const FLAC__
 static char *prepend(char *before, char *after)
     {
     char *new;
-    
+
     if (!(new = malloc(strlen(before) + strlen(after) + 1)))
         {
         fprintf(stderr, "malloc failure\n");
         return NULL;
         }
-        
+
     strcpy(new, before);
     strcat(new, after);
     free(after);
@@ -201,9 +201,9 @@ static void live_oggflac_encoder_main(struct encoder *encoder)
             int nmeta = 0, i = 0;
             size_t dlen = 0;
             FLAC__StreamMetadata_VorbisComment *vc;
-            
+
             live_ogg_capture_metadata(encoder, t);
-            
+
             if (t->custom && t->custom[0])
                 {
                 PREPEND("title=", t->custom)
@@ -226,7 +226,7 @@ static void live_oggflac_encoder_main(struct encoder *encoder)
                         fprintf(stderr, "live_oggflac_encoder_main: malloc failure\n");
                         goto bailout;
                         }
-                    
+
                 vc = &s->metadata[0]->data.vorbis_comment;
                 vc->num_comments = nmeta;
                 vc->vendor_string.entry = (FLAC__byte *)FLAC__VENDOR_STRING;
@@ -234,9 +234,9 @@ static void live_oggflac_encoder_main(struct encoder *encoder)
                 s->metadata[0]->type = FLAC__METADATA_TYPE_VORBIS_COMMENT;
                 s->metadata[0]->is_last = TRUE;
                 s->metadata[0]->length = nmeta * 4 + dlen + 8;
-                
+
                 vc->comments = realloc(vc->comments, nmeta * sizeof (FLAC__StreamMetadata_VorbisComment_Entry));
-                
+
                 APPEND(t->custom)
                 APPEND(t->artist)
                 APPEND(t->title)
@@ -283,7 +283,7 @@ static void live_oggflac_encoder_main(struct encoder *encoder)
             }
         return;
         }
-        
+
     if (encoder->encoder_state == ES_STOPPING)
         {
         FLAC__stream_encoder_delete(s->enc);
@@ -296,7 +296,7 @@ static void live_oggflac_encoder_main(struct encoder *encoder)
 
     fprintf(stderr, "live_oggflac_encoder_main: unhandled encoder state\n");
     return;
-    
+
     bailout:
     fprintf(stderr, "live_oggflac_encoder_main: performing cleanup\n");
     encoder->run_request_f = FALSE;
@@ -308,7 +308,7 @@ static void live_oggflac_encoder_main(struct encoder *encoder)
     if (s)
         {
         fprintf(stderr, "Clipping detected on upper %d times and lower %d times.\n", s->uclip, s->lclip);
-        
+
         if (s->metadata[0])
             {
             if (s->metadata[0]->data.vorbis_comment.comments)
@@ -318,7 +318,7 @@ static void live_oggflac_encoder_main(struct encoder *encoder)
         live_ogg_free_metadata(t);
         free(s);
         }
-    
+
     fprintf(stderr, "live_oggflac_encoder_main: finished cleanup\n");
     return;
     }
