@@ -39,7 +39,7 @@ int oggdec_get_next_packet(struct oggdec_vars *self)
     char *buffer;
     size_t bytes;
     int retval;
-    
+
     while ((retval = ogg_stream_packetout(&self->os, &self->op)) == 0)
         {
         while (ogg_sync_pageout(&self->oy, &self->og) != 1)
@@ -62,12 +62,12 @@ int oggdec_get_next_packet(struct oggdec_vars *self)
             if ((self->new_oggpage_callback))
                 self->new_oggpage_callback(self, self->new_oggpage_cb_userdata);
         }
-    
+
     if (retval == -1)
         {
         fprintf(stderr, "get_next_packet: hole in data detected - possibly not serious\n");
         }
-    
+
     return 1;
     }
 
@@ -83,7 +83,6 @@ static unsigned vorbis_get_samplerate(struct oggdec_vars *self)  /* attempt to g
     void obtain_tag_info(char *name, char **target, int multiple)
         {
         int tags = vorbis_comment_query_count(&vc, name);
-        int size, i;
 
         if (tags == 0)
             {
@@ -100,8 +99,8 @@ static unsigned vorbis_get_samplerate(struct oggdec_vars *self)  /* attempt to g
         if (multiple)
             {
             /* calculate the space needed */
-            size = tags;
-            for (i = 0; i < tags; i++)
+            size_t size = (size_t)tags;
+            for (int i = 0; i < tags; i++)
                 size += strlen(vorbis_comment_query(&vc, name, i));
 
             if (!(*target = malloc(size)))
@@ -113,7 +112,7 @@ static unsigned vorbis_get_samplerate(struct oggdec_vars *self)  /* attempt to g
             *target[0] = '\0';
 
             /* collect a slash separated list of tags */
-            for (i = 0; i < tags; i++)
+            for (int i = 0; i < tags; i++)
                 {
                 strcat(*target, vorbis_comment_query(&vc, name, i));
                 if (i < tags - 1)
@@ -137,7 +136,7 @@ static unsigned vorbis_get_samplerate(struct oggdec_vars *self)  /* attempt to g
         {
         samplerate = self->samplerate[self->ix] = vi.rate;
         self->channels[self->ix] = vi.channels;
-        
+
         if (vorbis_comment_query_count(&vc, "trk-title"))
             {
             obtain_tag_info("trk-artist", &self->artist[self->ix], TRUE);
@@ -150,7 +149,7 @@ static unsigned vorbis_get_samplerate(struct oggdec_vars *self)  /* attempt to g
             obtain_tag_info("title", &self->title[self->ix], TRUE);
             obtain_tag_info("album", &self->album[self->ix], TRUE);
             }
-      
+
         obtain_tag_info("replaygain_track_gain", &self->replaygain[self->ix], FALSE);
         obtain_tag_info("replaygain_reference_loudness", &self->rgloudness[self->ix], FALSE);
 
@@ -161,13 +160,13 @@ static unsigned vorbis_get_samplerate(struct oggdec_vars *self)  /* attempt to g
         samplerate = 0;
         self->channels[self->ix] = 0;
         }
-    
+
     vorbis_comment_clear(&vc);
     vorbis_info_clear(&vi);
 
     return samplerate;
     }
-    
+
 #ifdef HAVE_OGGFLAC
 
 FLAC__StreamDecoderReadStatus oggflac_read_callback(const FLAC__StreamDecoder *decoder, FLAC__byte buffer[], size_t *bytes, void *client_data)
@@ -190,10 +189,10 @@ FLAC__StreamDecoderReadStatus oggflac_read_callback(const FLAC__StreamDecoder *d
 
     if (ferror(self->fp))
         return FLAC__STREAM_DECODER_READ_STATUS_ABORT;
-        
+
     if (*bytes == 0)
         return FLAC__STREAM_DECODER_READ_STATUS_END_OF_STREAM;
-        
+
     return FLAC__STREAM_DECODER_READ_STATUS_CONTINUE;
     }
 
@@ -203,7 +202,7 @@ FLAC__StreamDecoderSeekStatus oggflac_seek_callback(const FLAC__StreamDecoder *d
     off_t start_bound, end_bound;
 
     start_bound = self->bos_offset[self->ix];
-    
+
     if (self->ix == self->n_streams - 1)
         end_bound = self->eos_offset - start_bound;
     else
@@ -214,7 +213,7 @@ FLAC__StreamDecoderSeekStatus oggflac_seek_callback(const FLAC__StreamDecoder *d
         fprintf(stderr, "oggflac_seek_callback: seek error1\n");
         return FLAC__STREAM_DECODER_SEEK_STATUS_ERROR;
         }
-    
+
     if (fseeko(self->fp, start_bound + (off_t)absolute_byte_offset, SEEK_SET) < 0)
         {
         fprintf(stderr, "oggflac_seek_callback: seek error2\n");
@@ -228,12 +227,12 @@ FLAC__StreamDecoderTellStatus oggflac_tell_callback(const FLAC__StreamDecoder *d
     {
     struct oggdec_vars *self = client_data;
     off_t where;
-    
+
     where = ftello(self->fp);
-    
+
     if (where < self->bos_offset[self->ix])
         return FLAC__STREAM_DECODER_TELL_STATUS_ERROR;
-    
+
     if (self->ix != self->n_streams - 1)
         {
         if (where > self->bos_offset[self->ix + 1])
@@ -244,7 +243,7 @@ FLAC__StreamDecoderTellStatus oggflac_tell_callback(const FLAC__StreamDecoder *d
         if (where > self->eos_offset)
             return FLAC__STREAM_DECODER_TELL_STATUS_ERROR;
         }
-    
+
     *absolute_byte_offset = (FLAC__uint64)(where - self->bos_offset[self->ix]);
     return FLAC__STREAM_DECODER_TELL_STATUS_OK;
     }
@@ -252,15 +251,15 @@ FLAC__StreamDecoderTellStatus oggflac_tell_callback(const FLAC__StreamDecoder *d
 FLAC__StreamDecoderLengthStatus oggflac_length_callback(const FLAC__StreamDecoder *decoder, FLAC__uint64 *stream_length, void *client_data)
     {
     struct oggdec_vars *self = client_data;
-    
+
     if (self->ix == self->n_streams - 1)
         *stream_length = self->eos_offset - self->bos_offset[self->ix];
     else
         *stream_length = self->bos_offset[self->ix + 1] - self->bos_offset[self->ix];
-        
+
     return FLAC__STREAM_DECODER_LENGTH_STATUS_OK;
     }
-    
+
 FLAC__bool oggflac_eof_callback(const FLAC__StreamDecoder *decoder, void *client_data)
     {
     struct oggdec_vars *self = client_data;
@@ -279,24 +278,29 @@ static void oggflac_metadata_callback(const FLAC__StreamDecoder *decoder, const
     const FLAC__StreamMetadata_StreamInfo *si;
     const FLAC__StreamMetadata_VorbisComment *vc;
     int use_alt_tags;
-    
+
     int match(char *t, char *comment)
         {
         return !strncasecmp(t, comment, strlen(t));
         }
-    
+
     char *end(char *t)
         {
-        while (*t++ != '=');
-        while (isspace(*t) && t != '\0')
-            t++;
+        while (*t != '=') {
+            // consider possibility of a tag that does not have an '='
+            if (*t == '\0')
+                return t;
+            ++t;
+        }
+        // eliminate whitespace following '=' character
+        while (isspace(*(++t)));
         return t;
         }
 
     void copy_tag(char *t, char **target, int multiple)
         {
         char *old, *new;
-        
+
         for (unsigned j = 0; j < vc->num_comments; j++)
             {
             if (match(t, (char *)vc->comments[j].entry))
@@ -314,7 +318,7 @@ static void oggflac_metadata_callback(const FLAC__StreamDecoder *decoder, const
         if (*target == NULL)
             *target = strdup("");
         }
-    
+
     if (metadata->type == FLAC__METADATA_TYPE_STREAMINFO)
         {
         fprintf(stderr, "oggflac_metadata_callback: got streaminfo metadata block\n");
@@ -362,7 +366,7 @@ static FLAC__StreamDecoderWriteStatus oggflac_write_callback(const FLAC__StreamD
     {
     return FLAC__STREAM_DECODER_WRITE_STATUS_CONTINUE;
     }
-    
+
 void oggflac_error_callback(const FLAC__StreamDecoder *decoder, FLAC__StreamDecoderErrorStatus se, void *client_data)
     {
     switch (se)
@@ -390,11 +394,11 @@ static int flac_get_samplerate(struct oggdec_vars *self)
         fprintf(stderr, "flac_get_samplerate: call to FLAC__stream_decoder_new failed\n");
         return 0;
         }
-    
+
     FLAC__stream_decoder_set_metadata_respond(decoder, FLAC__METADATA_TYPE_VORBIS_COMMENT);
-    
+
     if (FLAC__stream_decoder_init_ogg_stream(decoder,
-        oggflac_read_callback, oggflac_seek_callback, 
+        oggflac_read_callback, oggflac_seek_callback,
         oggflac_tell_callback, oggflac_length_callback,
         oggflac_eof_callback,  oggflac_write_callback,
         oggflac_metadata_callback, oggflac_error_callback,
@@ -404,7 +408,7 @@ static int flac_get_samplerate(struct oggdec_vars *self)
         FLAC__stream_decoder_delete(decoder);
         return 0;
         }
-        
+
     FLAC__stream_decoder_process_until_end_of_metadata(decoder);
     FLAC__stream_decoder_delete(decoder);
 
@@ -456,7 +460,7 @@ static int speex_get_samplerate(struct oggdec_vars *self)
                     }
                 else
                     return 0;
-                
+
                 return self->samplerate[self->ix];
             default:
                 speex_header_free(h);
@@ -481,7 +485,7 @@ static int opus_get_samplerate(struct oggdec_vars *self)
     unsigned initial_granulepos, final_granulepos;
     uint16_t preskip;
     char const *reason;
-    
+
     #define FAIL(x) do {reason = x; goto fail_point;} while(0)
     #define WARN(x) do {fprintf(stderr, "opus_get_samplerate: warning: %s\n", x);} while (0)
 
@@ -498,13 +502,13 @@ static int opus_get_samplerate(struct oggdec_vars *self)
 
         if (self->op.bytes < 19)
             FAIL("packet too small to be version 1");
-            
+
         if (self->op.packet[8] > 15)
             FAIL("encapsulation version unsupported");
-            
+
         if ((channels = ((unsigned char *)self->op.packet)[9]) == 0)
             FAIL("number of channels is zero");
-        
+
         self->channels[self->ix] = (channels == 1) ? 1 : 2;
         chanmap = ((unsigned char *)self->op.packet)[18];
 
@@ -521,7 +525,7 @@ static int opus_get_samplerate(struct oggdec_vars *self)
             {
             if (self->op.bytes != 21 + channels)
                 FAIL("OpusHead packet size wrong");
-            
+
             streamcount = ((unsigned char *)self->op.packet)[19];
             streamcount_2c = ((unsigned char *)self->op.packet)[20];
             if (streamcount == 0)
@@ -530,7 +534,7 @@ static int opus_get_samplerate(struct oggdec_vars *self)
                 FAIL("two channel streamcount > total streamcount");
             if (streamcount_2c + streamcount > 255)
                 FAIL("combined streamcount quantity exceeds 255");
-                
+
             unsigned char *cm = self->op.packet + 21;
             int index;
             for (int i = 0; i < channels; ++i)
@@ -578,7 +582,7 @@ static int opus_get_samplerate(struct oggdec_vars *self)
                     int track_gain_tags = vtag_comment_count(tag, "R128_TRACK_GAIN");
                     char *track_gain_text = vtag_lookup(tag, "R128_TRACK_GAIN", VLM_FIRST, NULL);
                     vtag_cleanup(tag);
-                    
+
                     switch (track_gain_tags)
                         {
                         case 0:
@@ -586,19 +590,20 @@ static int opus_get_samplerate(struct oggdec_vars *self)
                         case 1:
                             if (!track_gain_text)
                                 FAIL("vtag lookup failure");
-                            
+
                             if (isspace(track_gain_text[0]))
                                 FAIL("R128_TRACK_GAIN contains whitespace at start");
 
                             char *endp;
                             long value = strtol(track_gain_text, &endp, 10);
                             int endchar = *endp;
+                            int value_len = endp - track_gain_text;
                             free(track_gain_text);
-                            
+
                             if (endchar)
                                 FAIL("R128_TRACK_GAIN contains non digit data");
-                                
-                            if (endp - track_gain_text > 6 || value < -32768 || value > 32767)
+
+                            if (value_len > 6 || value < -32768 || value > 32767)
                                 FAIL("R128_TRACK_GAIN value out of range");
 
                             break;
@@ -615,7 +620,7 @@ static int opus_get_samplerate(struct oggdec_vars *self)
             }
         else
             FAIL("failed to get OpusTags packet");
-            
+
         if (oggdec_get_next_packet(self))
             {
             if ((frames = opus_packet_get_nb_frames(self->op.packet, self->op.bytes)) < 1)
@@ -627,10 +632,10 @@ static int opus_get_samplerate(struct oggdec_vars *self)
                 oggdec_get_next_packet(self);
                 if ((frames = opus_packet_get_nb_frames(self->op.packet, self->op.bytes)) < 1)
                     FAIL("packet with no frames detected");
-                
+
                 samples += opus_packet_get_samples_per_frame(self->op.packet, 48000) * frames;
                 }
-            
+
             if (self->op.e_o_s)
                 {
                 if (samples - self->op.granulepos > packetsamples)
@@ -650,13 +655,13 @@ static int opus_get_samplerate(struct oggdec_vars *self)
                     else
                         {
                         unsigned int pages_missing = initial_granulepos / samples;
-                        
+
                         if (ogg_page_pageno(&self->og) - pages_missing != 2)
                             WARN("opus_get_samplerate: ogg page numbering granulepos mismatch");
                         else
                             fprintf(stderr, "there are %u ogg pages missing -- this is normal for a captured stream\n", pages_missing);
                         }
-                        
+
                     if (samples - (final_granulepos - initial_granulepos) % samples > packetsamples)
                         WARN("end trimming > size of one packet");
                     }
@@ -690,12 +695,12 @@ static off_t oggscan_eos(struct oggdec_vars *self, off_t offset, off_t offset_en
     off_t  midpoint = (offset_end - offset) / 2 + offset;
     off_t  stored_mid = midpoint;
     int eos = FALSE, terminate = FALSE;
-    
+
     if (++depth >= 40)
         {
         fprintf(stderr, "maximum recursion depth %d reached on oggscan_eos\n", depth);
         return -1;
-        } 
+        }
 
     fseeko(self->fp, midpoint, SEEK_SET);
     ogg_sync_reset(&self->oy);
@@ -725,7 +730,7 @@ static off_t oggscan_eos(struct oggdec_vars *self, off_t offset, off_t offset_en
                 }
             }
         }
- 
+
     if (terminate || ogg_page_serialno(&self->og) == serial)
         {
         if (terminate || (eos = ogg_page_eos(&self->og)) || offset + 1 >= offset_end)
@@ -758,15 +763,15 @@ static off_t oggscan_eos(struct oggdec_vars *self, off_t offset, off_t offset_en
                 self->n_streams = 0;
                 return -1;
                 }
-          
+
             self->initial_granulepos[self->n_streams - 1] = 0;
             self->final_granulepos[self->n_streams - 1] = ogg_page_granulepos(&self->og);
-            self->serial[self->n_streams - 1] = serial; 
+            self->serial[self->n_streams - 1] = serial;
             if (!eos)
                 fprintf(stderr, "oggscan_eos: an unterminated stream was detected\n");
             return midpoint + retval;
             }
-        
+
         /* seek to the right next time */
         return oggscan_eos(self, midpoint, offset_end, serial, depth);
         }
@@ -789,9 +794,9 @@ static off_t oggscan(struct oggdec_vars *self, off_t *offset, off_t offset_end)
     size_t bytes;
     int    serial;
     off_t  retval;
-    
+
     fseeko(self->fp, *offset, SEEK_SET);
-    
+
     ogg_sync_reset(&self->oy);
     while ((retval = ogg_sync_pageseek(&self->oy, &self->og)) <= 0 || ogg_page_bos(&self->og) == 0)
         {
@@ -824,16 +829,16 @@ static struct oggdec_vars *oggdecode_get_metadata(char *pathname)
     int i;
     unsigned samplerate = 0;
     double start_time = 0.0;
-    
+
     /* allocate storage space */
     if (!(self = calloc(1, sizeof (struct oggdec_vars))))
         {
         fprintf(stderr, "oggdecode_reg: malloc failure\n");
         return NULL;
         }
-    
+
     self->magic = 4747;
-    
+
     /* open the media file */
     if (!(self->fp = fopen(pathname, "r")))
         {
@@ -864,7 +869,7 @@ static struct oggdec_vars *oggdecode_get_metadata(char *pathname)
         free(self);
         return NULL;
         }
-        
+
     if (ogg_stream_init(&self->os, 0))
         {
         fprintf(stderr, "oggdecode_reg: call to ogg_stream_init failed\n");
@@ -880,7 +885,7 @@ static struct oggdec_vars *oggdecode_get_metadata(char *pathname)
     while (offset < offset_end)
         {
         offset_new = oggscan(self, &offset, offset_end);
-        
+
         if (offset_new == -1)
             break;
         self->bos_offset[self->n_streams -1] = offset;
@@ -979,7 +984,7 @@ static struct oggdec_vars *oggdecode_get_metadata(char *pathname)
 static void oggdecode_free_metadata(struct oggdec_vars *self)
     {
     int i;
-    
+
     ogg_stream_clear(&self->os);
     ogg_sync_clear(&self->oy);
     fclose(self->fp);
@@ -994,7 +999,7 @@ static void oggdecode_free_metadata(struct oggdec_vars *self)
             if (self->album[i])
                 free(self->album[i]);
             }
-            
+
         free(self->bos_offset);
         free(self->initial_granulepos);
         free(self->final_granulepos);
@@ -1006,7 +1011,7 @@ static void oggdecode_free_metadata(struct oggdec_vars *self)
         free(self->start_time);
         free(self->duration);
         }
-    
+
     free(self);
     }
 
@@ -1018,7 +1023,7 @@ void oggdecode_seek_to_packet(struct oggdec_vars *self)
     ogg_int64_t granulepos = 0;
     char *buffer;
     size_t bytes;
-     
+
     start = self->bos_offset[self->ix];
     if (self->ix == self->n_streams - 1)
         end = self->eos_offset;
@@ -1033,7 +1038,7 @@ void oggdecode_seek_to_packet(struct oggdec_vars *self)
         ogg_sync_reset(&self->oy);
 
         for (;;)
-            { 
+            {
             while ((retval = ogg_sync_pageseek(&self->oy, &self->og)) <= 0)
                 {
                 if (retval < 0)
@@ -1074,13 +1079,13 @@ void oggdecode_dynamic_dispatcher(struct xlplayer *xlplayer)
     {
     struct oggdec_vars *s = xlplayer->dec_data;
     int success = 0, delay;
-    
+
     if (xlplayer->write_deferred)
         {
         xlplayer_write_channel_data(xlplayer);
         return;
         }
-    
+
     while (s->ix < s->n_streams)
         {
         /* skip over empty (read unplayable) streams */
@@ -1120,7 +1125,7 @@ void oggdecode_dynamic_dispatcher(struct xlplayer *xlplayer)
                 delay = xlplayer_calc_rbdelay(xlplayer);
             else
                 delay = 0;
-            
+
             if (s->artist[s->ix][0] || s->title[s->ix][0])
                 xlplayer_set_dynamic_metadata(xlplayer, DM_SPLIT_U8, s->artist[s->ix], s->title[s->ix], s->album[s->ix], delay);
             else
@@ -1128,7 +1133,7 @@ void oggdecode_dynamic_dispatcher(struct xlplayer *xlplayer)
                 fprintf(stderr, "oggdecode_dynamic_dispatcher: insufficient metadata\n");
                 xlplayer_set_dynamic_metadata(xlplayer, DM_NOTAG, "", "", "", delay);
                 }
-            
+
             xlplayer->usedelay = TRUE;
             return;
             }
@@ -1139,7 +1144,7 @@ void oggdecode_dynamic_dispatcher(struct xlplayer *xlplayer)
             s->ix++;
             }
         }
- 
+
     bugout:
     xlplayer->playmode = PM_EJECTING;
     }
@@ -1158,7 +1163,7 @@ static void oggdecode_init(struct xlplayer *xlplayer)
     {
     struct oggdec_vars *self = xlplayer->dec_data;
     int i;
-    
+
     /* calculate where we seek to */
     for (i = 0; i < self->n_streams; i++)
         {
@@ -1210,7 +1215,7 @@ int oggdecode_reg(struct xlplayer *xlplayer)
         xlplayer->dec_init = oggdecode_init;
         xlplayer->dec_play = oggdecode_dynamic_dispatcher;
         xlplayer->dec_eject = oggdecode_eject;
-        
+
         return ACCEPTED;
         }
     }
@@ -1219,13 +1224,13 @@ int oggdecode_get_metainfo(char *pathname, char **artist, char **title, char **a
     {
     struct oggdec_vars *self;
     int has_pbtime;
-    
+
     if(!(self = oggdecode_get_metadata(pathname)))
         {
         fprintf(stderr, "call to oggdecode_get_metadata failed for %s\n", pathname);
         return REJECTED;
         }
-        
+
     if ((has_pbtime = (*length = self->total_duration)))
         {
         if (self->n_streams > 1 && self->duration[0] > 0.1)
@@ -1252,9 +1257,9 @@ int oggdecode_get_metainfo(char *pathname, char **artist, char **title, char **a
                 *artist = realloc(*artist, 1);
                 *artist[0] = '\0';
                 }
-        
+
             if (self->title[0])
-                { 
+                {
                 if (*title)
                     free(*title);
                 *title = strdup(self->title[0]);
@@ -1266,7 +1271,7 @@ int oggdecode_get_metainfo(char *pathname, char **artist, char **title, char **a
                 }
 
             if (self->album[0])
-                { 
+                {
                 if (*album)
                     free(*album);
                 *album = strdup(self->album[0]);
