@@ -211,7 +211,7 @@ int streamer_connect(struct threads_info *ti, struct universal_vars *uv, void *o
     {
     struct streamer_vars *sv = other;
     struct streamer *self = ti->streamer[uv->tab];
-    int protocol, data_format = -1, tls;
+    int protocol, data_format = -1;
     char channels[2];
     char bitrate[6];
     char samplerate[6];
@@ -402,7 +402,9 @@ int streamer_connect(struct threads_info *ti, struct universal_vars *uv, void *o
         sce("make public");
         goto error;
         }
-    #if SHOUT_TLS
+    #ifdef HAVE_SHOUT_TLS
+    int tls = SHOUT_TLS_DISABLED;
+
     if (!strcmp("Disabled", sv->tls))
         tls = SHOUT_TLS_DISABLED;
     else if (!strcmp("Auto", sv->tls))
@@ -438,7 +440,7 @@ int streamer_connect(struct threads_info *ti, struct universal_vars *uv, void *o
         sce("client certificate");
         goto error;
         }
-    #endif
+    #endif /* HAVE_SHOUT_TLS */
 
     snprintf(channels,   sizeof channels  , "%d",  self->encoder_op->encoder->n_channels);
     {
