@@ -135,7 +135,7 @@ class IRCEntry(Gtk.Entry):  # pylint: disable=R0904
         """Builds the right click pop-up menu on the IRCEntry widget."""
 
         # TC: Popup menu item for a GTK text entry widget.
-        menuitem = Gtk.MenuItem(_('Insert Attribute or Colour Code'))
+        menuitem = Gtk.MenuItem(_('Insert Attribute or Color Code'))
         menu.append(menuitem)
         submenu = Gtk.Menu()
         menuitem.set_submenu(submenu)
@@ -183,7 +183,7 @@ class IRCEntry(Gtk.Entry):  # pylint: disable=R0904
         """Adder for menuitems that choose text colour."""
 
         for lower, upper in ((0, 7), (8, 15)):
-            menuitem = Gtk.MenuItem("{} {}-{}".format(_('Colours'), lower, upper))
+            menuitem = Gtk.MenuItem("{} {}-{}".format(_('Colors'), lower, upper))
             submenu.append(menuitem)
             colourmenu = Gtk.Menu()
             menuitem.set_submenu(colourmenu)
@@ -630,10 +630,10 @@ class MessageDialog(ChannelsDialog):
         self.message = IRCEntry()
         hbox.pack_start(l, False)
         hbox.pack_start(self.message)
-        set_tip(self.message, _("The message to send.\n\nOn the pop-up window "
+        set_tip(self.message, _("The message to send.\n\nIn the context menu "
         "(mouse right click) are some useful options for embedding metadata and"
         " for text formatting.\n\nThe window below displays how the message "
-        "will appear to users of XChat."))
+        "will appear within IRC chat clients such as HexChat."))
         self.mainbox.pack_start(hbox, False)
 
         sw = Gtk.ScrolledWindow()
