@@ -1,5 +1,5 @@
 #   mutagengui.py: GTK based file tagger.
-#   Copyright (C) 2009 Stephen Fairchild (s-fairchild@users.sourceforge.net)
+#   Copyright (C) 2009-2020 Stephen Fairchild (s-fairchild@users.sourceforge.net)
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -15,8 +15,6 @@
 #   along with this program in the file entitled COPYING.
 #   If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import print_function
-
 __all__ = ['MutagenGUI']
 
 import os
@@ -25,9 +23,9 @@ import string
 import re
 import gettext
 
-import gtk
-import pango
-import glib
+from gi.repository import Gtk
+from gi.repository import Pango
+from gi.repository import GLib
 import mutagen
 import mutagen.id3 as id3
 from mutagen.mp3 import MP3
@@ -40,71 +38,87 @@ from idjc import FGlobs
 from .tooltips import set_tip
 from idjc.prelims import ProfileManager
 
-
 t = gettext.translation(FGlobs.package_name, FGlobs.localedir, fallback=True)
 _ = t.gettext
 
 pm = ProfileManager()
 
+def write_error_dialog(error, window):
+    dialog = Gtk.Dialog(title=_('Tag Write Failed'))
+    dialog.set_modal(True)
+    dialog.set_transient_for(window)
+    dialog.set_border_width(8)
+    dialog.set_size_request(300, 60)
+    content_vbox = dialog.get_content_area()
+    content_vbox.set_spacing(10)
+    label = Gtk.Label(str(error))
+    label.set_max_width_chars(40)
+    label.set_line_wrap(True)
+    label.set_single_line_mode(False)
+    label.set_selectable(True)
+    content_vbox.add(label)
+    label.show()
+    dialog.add_button(_("OK"), Gtk.ResponseType.OK)
+    dialog.run()
+    dialog.destroy()
 
 
-class LeftLabel(gtk.HBox):
-    """Use in place of gtk.Label where left justification is needed."""
-    
-    def __init__(self, text):
-        gtk.HBox.__init__(self)
-        self.label = gtk.Label(text)
-        self.pack_start(self.label, False, False, 0)
+def mono_label(text):
+    label = Gtk.Label.new(f"<span font-family='monospace'>{text}</span>")
+    label.set_use_markup(True)
+    label.set_margin_top(2)
+    return label
 
 
-
-class RightLabel(gtk.HBox):
-    """Use in place of gtk.Label where right justification is needed."""
-    
-    def __init__(self, text):
-        gtk.HBox.__init__(self)
-        self.pack_end(gtk.Label(text), False, False, 0)
+def right_label(text):
+    label = Gtk.Label.new(text)
+    label.set_xalign(1.0)
+    return label
 
 
-
-class FreeTagFrame(gtk.Frame):
+class FreeTagFrame(Gtk.Frame):
     def __init__(self):
-        gtk.Frame.__init__(self)
-        sw = gtk.ScrolledWindow()
+        Gtk.Frame.__init__(self)
+        sw = Gtk.ScrolledWindow()
         sw.set_border_width(5)
-        sw.set_policy(gtk.POLICY_NEVER, gtk.POLICY_ALWAYS)
+        sw.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.ALWAYS)
         self.add(sw)
         sw.show()
-        self.tb = gtk.TextBuffer()
-        tv = gtk.TextView(self.tb)
-        tv.set_wrap_mode(gtk.WRAP_CHAR)
-        tv.modify_font(pango.FontDescription('sans 12'))
-        sw.add(tv)
-        tv.show()
+
+        text_view = Gtk.TextView()
+        text_view.set_wrap_mode(Gtk.WrapMode.CHAR)
+        self.text_buffer = text_view.get_buffer()
+        text_tag = self.text_buffer.create_tag("font", font="monospace 11")
+        sw.add(text_view)
+        text_view.show()
+        self.text_buffer.connect("changed", self._on_buffer_changed)
+
+    def _on_buffer_changed(self, text_buffer):
+        """Keep all text in the same font."""
+
+        text_buffer.apply_tag_by_name("font", *text_buffer.get_bounds())
 
 
-
-class MutagenTagger(gtk.VBox):
+class MutagenTagger(Gtk.VBox):
     """Base class for ID3Tagger and NativeTagger."""
-    
-    def __init__(self, pathname):
-        gtk.VBox.__init__(self)
-        self.pathname = pathname
 
+    def __init__(self, pathname):
+        Gtk.VBox.__init__(self)
+        self.pathname = pathname
 
 
 class WMATagger(MutagenTagger):
     """Handles tagging of WMA files"""
-    
+
     primary_data = ("Title", "Author")
     secondaries = ("WM/AlbumTitle", "WM/AlbumArtist", "WM/Year", "WM/Genre")
-    
-    def save_tag(self):
+
+    def save_tag(self, window):
         """Updates the tag with the GUI data."""
-         
+
         tag = self.tag
-        tb = self.tag_frame.tb
-         
+        tb = self.tag_frame.text_buffer
+
         for key in self.text_set:
             try:
                 del tag[key]
@@ -120,8 +134,8 @@ class WMATagger(MutagenTagger):
                     del tag[each[0]]
                 except KeyError:
                     pass
-                 
-        lines = tb.get_text(tb.get_start_iter(), tb.get_end_iter()).splitlines()
+
+        lines = tb.get_text(tb.get_start_iter(), tb.get_end_iter(), False).splitlines()
         for line in lines:
             try:
                 key, val = line.split("=", 1)
@@ -132,47 +146,46 @@ class WMATagger(MutagenTagger):
                 val = val.strip()
                 if val:
                     try:
-                        tag[key] += [ASFUnicodeAttribute(val.decode("utf-8"))]
+                        tag[key] += [ASFUnicodeAttribute(val)]
                     except (KeyError, AttributeError):
                         try:
                             tag[key] = [
-                                    ASFUnicodeAttribute(val.decode("utf-8"))]
+                                    ASFUnicodeAttribute(val)]
                         except KeyError:
                             print("Unacceptable key", key)
-        tag.save()
+        try:
+            tag.save()
+        except mutagen.MutagenError as e:
+            write_error_dialog(e, window)
 
-    
     def load_tag(self):
         """(re)Writes the tag data to the GUI."""
-        
+
         tag = self.tag
-        
+
         for each in self.primary_line:
             try:
                 data = tag[each[0]]
             except KeyError:
                 pass
             else:
-                each[1].set_text("/".join(unicode(y) for y in data))
+                each[1].set_text("/".join(y.value for y in data))
 
         additional = []
 
         for key in self.secondaries:
             values = tag.get(key, [ASFUnicodeAttribute("")])
             for val in values:
-                additional.append(key.encode("utf-8") + "=" + unicode(
-                                                        val).encode("utf-8"))
+                additional.append(f"{key}={val}")
 
         for key in self.text_set:
             if key not in self.primary_data and key not in self.secondaries:
                 values = tag[key]
                 for val in values:
-                    additional.append(key.encode("utf-8") + "=" + unicode(
-                                                        val).encode("utf-8"))
-        
-        self.tag_frame.tb.set_text("\n".join(additional))
+                    additional.append(f"{key}={val}")
+
+        self.tag_frame.text_buffer.set_text("\n".join(additional))
 
-    
     def __init__(self, pathname):
         MutagenTagger.__init__(self, pathname)
         try:
@@ -183,22 +196,21 @@ class WMATagger(MutagenTagger):
             print("Not a real wma/asf file apparently.")
             self.tag = None
             return
-            
-        hbox = gtk.HBox()
-        hbox.set_border_width(5)
-        hbox.set_spacing(8)
-        self.pack_start(hbox, False, False, 0)
-        vbox_text = gtk.VBox()
-        hbox.pack_start(vbox_text, False, False, 0)
-        vbox_entry = gtk.VBox()
-        hbox.pack_start(vbox_entry, True, True, 0)
-        
+
+        grid = Gtk.Grid()
+        grid.set_border_width(5)
+        grid.set_column_spacing(8)
+        grid.set_row_homogeneous(True)
+        self.add(grid)
+
         self.primary_line = []
-        for text, entry in ((x, gtk.Entry()) for x in self.primary_data):
+        for row, (text, entry) in enumerate(
+                            (x, Gtk.Entry()) for x in self.primary_data):
             self.primary_line.append((text, entry))
-            vbox_text.add(LeftLabel(text))
-            vbox_entry.add(entry)
-        hbox.show_all()
+            entry.set_hexpand(True)
+            grid.attach(right_label(text), 0, row, 1, 1)
+            grid.attach(entry, 1, row, 1, 1)
+        grid.show_all()
 
         self.tag_frame = FreeTagFrame()
         self.tag_frame.set_border_width(5)
@@ -207,70 +219,68 @@ class WMATagger(MutagenTagger):
 
         self.text_set = []
 
-        for key, val in self.tag.iteritems():
+        for key, val in self.tag.items():
             if key not in self.primary_line and all(isinstance(v, (
-                                ASFUnicodeAttribute, unicode)) for v in val):
+                                ASFUnicodeAttribute, str)) for v in val):
                 self.text_set.append(key)
 
 
 
 class ID3Tagger(MutagenTagger):
     """ID3 tagging with Mutagen."""
-    
+
     primary_data = (("TIT2", _('title')), ("TPE1", _('artist')),
                          ("TALB", _('album')), ("TRCK", _('track/total')),
                          ("TCON", _('genre')), ("TDRC", _('record date')))
 
-    
-    def save_tag(self):
+    def save_tag(self, window):
         """Updates the tag with the GUI data."""
-        
+
         tag = self.tag
-        
+
         # Remove all text tags.
-        for fid in tag.iterkeys():
-            if fid[0] == "T":
-                del tag[fid]
-    
+        for fid in [fid for fid in tag if fid[0] == 'T']:
+            del tag[fid]
+
         # Add the primary tags.
         for fid, entry in self.primary_line:
             text = entry.get_text().strip()
             if text:
                 frame = getattr(id3, fid)
                 tag[fid] = frame(3, [text])
- 
+
         # Add the freeform text tags.
-        tb = self.tag_frame.tb
-        lines = tb.get_text(tb.get_start_iter(), tb.get_end_iter()).splitlines()
-             
+        tb = self.tag_frame.text_buffer
+        lines = tb.get_text(tb.get_start_iter(), tb.get_end_iter(), False).splitlines()
+
         for line in lines:
             try:
                 fid, val = line.split(":", 1)
-            
+
             except ValueError:
                 continue
-            
+
             fid = fid.strip()
-            val = val.strip().decode("utf-8")
-            
+            val = val.strip()
+
             try:
                 frame = id3.Frames[fid]
             except NameError:
                 continue
- 
+
             if not issubclass(frame, id3.TextFrame):
                 continue
- 
+
             if frame is id3.TXXX:
                 try:
                     key, val = val.split(u"=", 1)
-                
+
                 except ValueError:
                     continue
-                
+
                 f = frame(3, key.strip(), [val.strip()])
                 tag[f.HashKey] = f
-                
+
             else:
                 try:
                     val_list = tag[fid].text
@@ -279,15 +289,18 @@ class ID3Tagger(MutagenTagger):
                 else:
                     val_list.append(val)
 
-        tag.save()
+        try:
+            tag.save()
+        except mutagen.MutagenError as e:
+            write_error_dialog(e, window)
+
 
-                
     def load_tag(self):
         """(re)Writes the tag data to the GUI."""
-        
+
         additional = []
         done = []
-        
+
         for fid, entry in self.primary_line:
             try:
                 frame = self.tag[fid]
@@ -298,21 +311,21 @@ class ID3Tagger(MutagenTagger):
                         # Handle occurrence of ID3Timestamp.
                         entry.set_text(str(frame.text[0]))
                     for each in frame.text[1:]:
-                        additional.append(fid + ":" + each.encode("utf-8"))
+                        additional.append(f"{fid}:{each}")
             except KeyError:
                 entry.set_text("")
-            
+
             done.append(fid)
-                
-        for fid, frame in self.tag.iteritems():
+
+        for fid, frame in self.tag.items():
             if fid[0] == "T" and fid not in done:
                 sep = "=" if fid.startswith("TXXX:") else ":"
                 for text in frame.text:
-                    additional.append(fid + sep + text.encode("utf-8"))
-                
-        self.tag_frame.tb.set_text("\n".join(additional))
+                    additional.append(f"{fid}{sep}"
+                                      f"{text if type(text) is str else text.text}")
+
+        self.tag_frame.text_buffer.set_text("\n".join(additional))
 
-        
     def __init__(self, pathname, force=False):
         MutagenTagger.__init__(self, pathname)
         if force:
@@ -336,28 +349,24 @@ class ID3Tagger(MutagenTagger):
             except mutagen.id3.error:
                 self.tag = None
                 return
-            
-        hbox = gtk.HBox()
-        hbox.set_border_width(5)
-        hbox.set_spacing(8)
-        self.pack_start(hbox, False, False, 0)
-        vbox_frame = gtk.VBox()
-        hbox.pack_start(vbox_frame, False, False, 0)
-        vbox_text = gtk.VBox()
-        hbox.pack_start(vbox_text, False, False, 0)
-        vbox_entry = gtk.VBox()
-        hbox.pack_start(vbox_entry, True, True, 0)
-        
+
+        grid = Gtk.Grid()
+        grid.set_border_width(5)
+        grid.set_column_spacing(8)
+        self.pack_start(grid, False, False, 0)
+
         self.primary_line = []
-        for frame, text, entry in (
-                            (x, y, gtk.Entry()) for x, y in self.primary_data):
+        for i, (frame, text, entry) in enumerate(
+                            (x, y, Gtk.Entry()) for x, y in self.primary_data):
             self.primary_line.append((frame, entry))
-            vbox_frame.add(LeftLabel(frame))
-            vbox_text.add(RightLabel(text))
-            vbox_entry.add(entry)
-        hbox.show_all()
-        
+            grid.attach(mono_label(frame), 0, i, 1, 1)
+            grid.attach(right_label(text), 1, i, 1, 1)
+            grid.attach(entry, 2, i, 1, 1)
+            entry.set_hexpand(True)
+        grid.show_all()
+
         self.tag_frame = FreeTagFrame()
+        self.tag_frame.set_vexpand(True)
         set_tip(self.tag_frame, _('Add any other ID3 text frames here.\ne.g. '
         'TIT2:Alternate Title\nThis will be appended onto the main TIT2 tag.'
         '\n\nEnter user defined text frames like this:\nTXXX:foo=bar\n\n'
@@ -369,29 +378,28 @@ class ID3Tagger(MutagenTagger):
         self.tag_frame.show()
 
 
-
 class MP4Tagger(MutagenTagger):
     """MP4 tagging with Mutagen."""
-    
+
     primary_data = (("\xa9nam", _('Title')), ("\xa9ART", _('Artist')),
                          ("\xa9alb", _('Album')), ("trkn", _('Track')),
                          ("\xa9gen", _('Genre')), ("\xa9day", _('Year')))
-    
-    def save_tag(self):
+
+    def save_tag(self, window):
         """Updates the tag with the GUI data."""
-        
+
         tag = self.tag
         for fid, entry in self.primary_line:
             text = entry.get_text().strip()
             if fid == "trkn":
-                mo1 = re.search("\d+", text)
+                mo1 = re.search(r"\d+", text)
                 try:
                     track = int(text[mo1.start():mo1.end()])
                 except AttributeError:
                     new_val = None
                 else:
                     text = text[mo1.end():]
-                    mo2 = re.search("\d+", text)
+                    mo2 = re.search(r"\d+", text)
                     try:
                         total = int(text[mo2.start():mo2.end()])
                     except AttributeError:
@@ -409,14 +417,17 @@ class MP4Tagger(MutagenTagger):
                 except KeyError:
                     pass
 
-        tag.save()
+        try:
+            tag.save()
+        except mutagen.MutagenError as e:
+            write_error_dialog(e, window)
+
 
-                
     def load_tag(self):
         """(re)Writes the tag data to the GUI."""
-        
+
         additional = []
-        
+
         for fid, entry in self.primary_line:
             try:
                 frame = self.tag[fid][0]
@@ -431,7 +442,6 @@ class MP4Tagger(MutagenTagger):
                 else:
                     entry.set_text(frame)
 
-             
     def __init__(self, pathname):
         MutagenTagger.__init__(self, pathname)
         try:
@@ -442,44 +452,39 @@ class MP4Tagger(MutagenTagger):
             print("Not a real mp4 file apparently.")
             self.tag = None
             return
-            
-        hbox = gtk.HBox()
-        hbox.set_border_width(5)
-        hbox.set_spacing(8)
-        self.pack_start(hbox, False, False, 0)
-        vbox_text = gtk.VBox()
-        hbox.pack_start(vbox_text, False, False, 0)
-        vbox_entry = gtk.VBox()
-        hbox.pack_start(vbox_entry, True, True, 0)
-        
-        self.primary_line = []
-        for frame, text, entry in (
-                            (x, y, gtk.Entry()) for x, y in self.primary_data):
-            self.primary_line.append((frame, entry))
-            vbox_text.add(LeftLabel(text))
-            vbox_entry.add(entry)
-        hbox.show_all()
 
+        grid = Gtk.Grid()
+        grid.set_border_width(5)
+        grid.set_column_spacing(8)
+        grid.set_row_homogeneous(True)
+        self.add(grid)
+
+        self.primary_line = []
+        for row, (frame, text, entry) in enumerate(
+                            (x, y, Gtk.Entry()) for x, y in self.primary_data):
+            self.primary_line.append((frame, entry))
+            entry.set_hexpand(True)
+            grid.attach(right_label(text), 0, row, 1, 1)
+            grid.attach(entry, 1, row, 1, 1)
+        grid.show_all()
 
 
 class NativeTagger(MutagenTagger):
     """Native format tagging with Mutagen. Mostly FLAC and Ogg."""
-    
+
     blacklist = "coverart", "metadata_block_picture"
 
-    
-    def save_tag(self):
+    def save_tag(self, window):
         """Updates the tag with the GUI data."""
-        
+
         tag = self.tag
-        
-        for key in tag.iterkeys():
-            if key not in self.blacklist:
-                del tag[key]
-                
-        tb = self.tag_frame.tb
-        lines = tb.get_text(tb.get_start_iter(), tb.get_end_iter()).splitlines()
-        
+
+        for key in [key for key in tag if key not in self.blacklist]:
+            del tag[key]
+
+        tb = self.tag_frame.text_buffer
+        lines = tb.get_text(tb.get_start_iter(), tb.get_end_iter(), False).splitlines()
+
         for line in lines:
             try:
                 key, val = line.split("=", 1)
@@ -490,40 +495,41 @@ class NativeTagger(MutagenTagger):
                 val = val.strip()
                 if key not in self.blacklist and val:
                     try:
-                        tag[key] += [val.decode("utf-8")]
+                        tag[key] += [val]
                     except (KeyError, AttributeError):
                         try:
-                            tag[key] = [val.decode("utf-8")]
+                            tag[key] = [val]
                         except KeyError:
                             print("Unacceptable key", key)
-    
-        tag.save() 
 
-    
+        try:
+            tag.save()
+        except mutagen.MutagenError as e:
+            write_error_dialog(e, window)
+
     def load_tag(self):
         """(re)Writes the tag data to the GUI."""
-        
+
         tag = self.tag
         lines = []
         primaries = "title", "artist", "author", "album",\
                                   "tracknumber", "tracktotal", "genre", "date"
-        
         for key in primaries:
             try:
                 values = tag[key]
             except KeyError:
-                lines.append(key + "=")
+                lines.append(f"{key}=")
             else:
                 for val in values:
-                    lines.append(key + "=" + val.encode("utf-8"))
+                    lines.append(f"{key}={val}")
 
-        for key, values in tag.iteritems():
+        for key, values in tag.items():
             if key not in primaries and key not in self.blacklist:
                 for val in values:
-                    lines.append(key + "=" + val.encode("utf-8"))
-                
-        self.tag_frame.tb.set_text("\n".join(lines))
-    
+                    lines.append(f"{key}={val}")
+
+        self.tag_frame.text_buffer.set_text("\n".join(lines))
+
 
     def __init__(self, pathname, ext):
         MutagenTagger.__init__(self, pathname)
@@ -532,8 +538,9 @@ class NativeTagger(MutagenTagger):
             # MP3 and APEv2 have their own specialised tagger.
             self.tag = None
             return
-        
+
         self.tag_frame = FreeTagFrame()
+        self.tag_frame.set_vexpand(True)
         self.add(self.tag_frame)
         self.tag_frame.show()
 
@@ -541,22 +548,20 @@ class NativeTagger(MutagenTagger):
 
 class ApeTagger(MutagenTagger):
     """APEv2 tagging with Mutagen."""
-    
+
     opener = {"ape": MonkeysAudio, "mpc": Musepack }
 
-    
-    def save_tag(self):
+    def save_tag(self, window):
         """Updates the tag with the GUI data."""
-        
+
         tag = self.tag
-        
-        for key, values in tag.iteritems():
-            if isinstance(values, APETextValue):
-                del tag[key]
-                
-        tb = self.tag_frame.tb
-        lines = tb.get_text(tb.get_start_iter(), tb.get_end_iter()).splitlines()
-        
+
+        for k in [k for k, v in tag.items() if isinstance(v, APETextValue)]:
+            del tag[k]
+
+        tb = self.tag_frame.text_buffer
+        lines = tb.get_text(tb.get_start_iter(), tb.get_end_iter(), False).splitlines()
+
         for line in lines:
             try:
                 key, val = line.split("=", 1)
@@ -567,44 +572,45 @@ class ApeTagger(MutagenTagger):
                 val = val.strip()
                 if val:
                     try:
-                        tag[key].value += "\0" + val.decode("utf-8")
+                        tag[key].value += "\0" + val
                     except (KeyError, AttributeError):
                         try:
-                            tag[key] = APETextValue(val.decode("utf-8"), 0)
+                            tag[key] = APETextValue(val, 0)
                         except KeyError:
                             print("Unacceptable key", key)
-    
-        tag.save() 
 
-    
+        try:
+            tag.save()
+        except mutagen.MutagenError as e:
+            write_error_dialog(e, window)
+
     def load_tag(self):
         """(re)Writes the tag data to the GUI."""
-        
+
         tag = self.tag
         lines = []
         primaries = "TITLE", "ARTIST", "AUTHOR", "ALBUM",\
                                   "TRACKNUMBER", "TRACKTOTAL", "GENRE", "DATE"
-        
+
         for key in primaries:
             try:
                 values = tag[key]
             except KeyError:
-                lines.append(key + "=")
+                lines.append(f"{key}=")
             else:
                 for val in values:
-                    lines.append(key + "=" + val.encode("utf-8"))
+                    lines.append(f"{key}={val}")
 
-        for key, values in tag.iteritems():
+        for key, values in tag.items():
             if key not in primaries and isinstance(values, APETextValue):
                 for val in values:
-                    lines.append(key + "=" + val.encode("utf-8"))
-                
-        self.tag_frame.tb.set_text("\n".join(lines))
+                    lines.append(f"{key}={val}")
+
+        self.tag_frame.text_buffer.set_text("\n".join(lines))
 
-    
     def __init__(self, pathname, extension):
         MutagenTagger.__init__(self, pathname)
-        
+
         try:
             self.tag = self.opener[extension](pathname)
         except KeyError:
@@ -627,8 +633,9 @@ class ApeTagger(MutagenTagger):
                 print("ape tag found on native format")
             else:
                 print("no existing ape tags found")
-            
+
         self.tag_frame = FreeTagFrame()
+        self.tag_frame.set_vexpand(True)
         self.add(self.tag_frame)
         self.tag_frame.show()
 
@@ -639,18 +646,15 @@ class MutagenGUI:
                 "flac": "FLAC", "ogg": "Ogg Vorbis", "oga": "XIPH Ogg audio", "opus": "Ogg Opus",
                 "m4b": "MP4", "m4p": "MP4", "wma": "Windows Media Audio"}
 
-    
     def destroy_and_quit(self, widget, data = None):
-        gtk.main_quit()
+        Gtk.main_quit()
         sys.exit(0)
 
-    
-    def update_playlists(self, pathname, idjcroot):
-        newplaylistdata = idjcroot.player_left.get_media_metadata(pathname)
-        idjcroot.player_left.update_playlist(newplaylistdata)
-        idjcroot.player_right.update_playlist(newplaylistdata)
+    def update_playlists(self, _, pathname, idjcroot):
+        idjcroot.player_left.update_playlist(pathname)
+        idjcroot.player_right.update_playlist(pathname)
+        idjcroot.background.player.update_playlist(pathname)
 
-    
     @staticmethod
     def is_supported(pathname):
         supported = [ "mp2", "mp3", "ogg", "oga" ]
@@ -670,77 +674,75 @@ class MutagenGUI:
         else:
             return extension
 
-    
     def __init__(self, pathname, encoding, idjcroot = None):
-        if not pathname:
-            print("Tagger not supplied any pathname.")
+        if not os.path.isfile(pathname or ""):
+            print(f"tagger: bad pathname: {pathname}")
             return
-        
+
         extension = self.is_supported(pathname)
         if extension == False:
-            print("Tagger file extension", extension, "not supported.")
+            print("tagger: file extension", extension, "not supported.")
             return
-        
-        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
+
+        self.window = Gtk.Window()
+        self.window.set_border_width(4)
         if idjcroot is not None:
             idjcroot.window_group.add_window(self.window)
         self.window.set_size_request(550, 450)
         # TC: Window title.
         self.window.set_title(_('IDJC Tagger') + pm.title_extra)
         self.window.set_destroy_with_parent(True)
-        self.window.set_border_width(9)
         self.window.set_resizable(True)
         if idjcroot == None:
             self.window.connect("destroy", self.destroy_and_quit)
-        vbox = gtk.VBox()
+        vbox = Gtk.VBox()
+        vbox.set_spacing(3)
         self.window.add(vbox)
         vbox.show()
-        label = gtk.Label()
-        if idjcroot:
-            if encoding is not None:
-                label.set_markup(u"<b>" + _('Filename:').decode("utf-8") + \
-                u" " + glib.markup_escape_text(unicode(os.path.split(
-                pathname)[1], encoding).encode("utf-8", "replace")) + u"</b>")
-            else:
-                label.set_markup(u"<b>" + _('Filename:').decode("utf-8") + \
-                u" " + glib.markup_escape_text(os.path.split(
-                pathname)[1]).encode("utf-8", "replace") + u"</b>")
-        else:
-            label.set_markup(u"<b>" + _('Filename:').decode("utf-8") + u" " + \
-            glib.markup_escape_text(unicode(os.path.split(
-            pathname)[1], "latin1").encode("utf-8", "replace")) + u"</b>")
+        label = Gtk.Label()
+        label.set_markup("<b>" + _('Filename:') + " " + \
+                         GLib.markup_escape_text(os.path.split(
+                             pathname)[1]) + "</b>")
         vbox.pack_start(label, False, False, 6)
         label.show()
-        
-        hbox = gtk.HBox()
-        hbox.set_border_width(2)
-        apply_button = gtk.Button(None, gtk.STOCK_APPLY)
-        if idjcroot is not None:
-            apply_button.connect_object_after("clicked", self.update_playlists,
-                                                            pathname, idjcroot)
-        hbox.pack_end(apply_button, False, False, 0)
-        apply_button.show()
-        close_button = gtk.Button(None, gtk.STOCK_CLOSE)
-        close_button.connect_object("clicked", gtk.Window.destroy, self.window)
-        hbox.pack_end(close_button, False, False, 10)
+
+        hbox = Gtk.HBox()
+        bb = Gtk.ButtonBox()
+        bb.set_spacing(3)
+        bb.set_layout(Gtk.ButtonBoxStyle.END)
+        hbox.pack_end(bb, True)
+        bb.show()
+
+        close_button = Gtk.Button.new_with_label(_("Close"))
+        close_button.connect_object("clicked", Gtk.Window.destroy, self.window)
+        bb.add(close_button)
         close_button.show()
-        reload_button = gtk.Button(None, gtk.STOCK_REVERT_TO_SAVED)
-        hbox.pack_start(reload_button, False, False, 10)
+
+        apply_button = Gtk.Button.new_with_label(_("Apply"))
+        if idjcroot is not None:
+            apply_button.connect_after("clicked", self.update_playlists,
+                                                            pathname, idjcroot)
+        bb.add(apply_button)
+        apply_button.show()
+
+        reload_button = Gtk.Button.new_from_icon_name("edit-undo-symbolic",
+                                                      Gtk.IconSize.BUTTON)
+        # reload_button.set_label(_("Undo"))
+        hbox.pack_start(reload_button, False, False, 0)
         reload_button.show()
         vbox.pack_end(hbox, False, False, 0)
         hbox.show()
-        hbox = gtk.HBox()
+        hbox = Gtk.HBox()
         vbox.pack_end(hbox, False, False, 2)
         hbox.show()
-        
-        notebook = gtk.Notebook()
-        notebook.set_border_width(2)
+
+        notebook = Gtk.Notebook()
         vbox.pack_start(notebook, True, True, 0)
         notebook.show()
-        
+
         try:
             self.ape = ApeTagger(pathname, extension)
-            
+
             if extension in ("mp3", "aac"):
                 self.id3 = ID3Tagger(pathname, True)
                 self.native = None
@@ -755,35 +757,35 @@ class MutagenGUI:
                     self.native = None
                 else:
                     self.native = NativeTagger(pathname, ext=extension)
-            
+
             if self.id3 is not None and self.id3.tag is not None:
                 reload_button.connect("clicked", lambda x: self.id3.load_tag())
-                apply_button.connect("clicked", lambda x: self.id3.save_tag())
-                label = gtk.Label("ID3")
+                apply_button.connect("clicked", lambda x: self.id3.save_tag(self.window))
+                label = Gtk.Label("ID3")
                 notebook.append_page(self.id3, label)
                 self.id3.show()
-            
+
             if self.ape is not None and self.ape.tag is not None:
                 reload_button.connect("clicked", lambda x: self.ape.load_tag())
-                apply_button.connect("clicked", lambda x: self.ape.save_tag())
-                label = gtk.Label("APE v2")
+                apply_button.connect("clicked", lambda x: self.ape.save_tag(self.window))
+                label = Gtk.Label("APE v2")
                 notebook.append_page(self.ape, label)
-                self.ape.show() 
-            
+                self.ape.show()
+
             if self.native is not None and self.native.tag is not None:
                 reload_button.connect("clicked",
                                             lambda x: self.native.load_tag())
                 apply_button.connect("clicked",
-                                            lambda x: self.native.save_tag())
-                label = gtk.Label(_('Native') + " (" + self.ext2name[
+                                            lambda x: self.native.save_tag(self.window))
+                label = Gtk.Label.new(_('Native') + " (" + self.ext2name[
                                                             extension] + ")")
                 notebook.append_page(self.native, label)
                 self.native.show()
-            
+
             reload_button.clicked()
 
             apply_button.connect_object_after("clicked",
-                                            gtk.Window.destroy, self.window)
+                                            Gtk.Window.destroy, self.window)
             self.window.show()
         except IOError as e:
             print(e)
