@@ -698,7 +698,7 @@ class PortGrabPage(Gtk.Grid):
         self.page_number = page_number
         self.textdict = {}
 
-        self.regex_toggle = Gtk.CheckButton(label=_('Enable regular expression matching.'))
+        self.regex_toggle = Gtk.CheckButton(label=_('Enable regular expression matching'))
         self.regex_toggle.set_border_width(3)
         self.activedict = {f"port_grab_rexex_{page_number}": self.regex_toggle}
         self.attach(self.regex_toggle, 0, 0, 1, 1)
@@ -946,6 +946,9 @@ class mixprefs:
         self.parent.grid1.remove(self.parent.history_vbox)
         self.parent.grid1.attach(self.parent.history_vbox, 0, 16, 1, int(widget.get_value()))
 
+    def cb_history_scale2(self, *args):
+        self.parent.history_expander.set_expanded(True)
+
     def cb_mic_boost(self, widget):
         self.parent.send_new_mixer_stats()
 
@@ -1124,7 +1127,7 @@ class mixprefs:
 
         # Not wanting to add a pane into the main player window as it would look ugly
         # we get this instead. Extreme values produce large application window sizes.
-        frame = Gtk.Frame.new(" {} ".format(_("History Window Relative Size")))
+        frame = Gtk.Frame.new(" {} ".format(_("Tracks Played Panel Size (relative to playlist size)")))
         frame.set_label_align(0.5, 0.5)
         frame.set_border_width(3)
         history_scale_hbox = Gtk.HBox()
@@ -1132,6 +1135,7 @@ class mixprefs:
         self.history_scale = Gtk.Scale.new_with_range(Gtk.Orientation.HORIZONTAL, 1.0, 24.0, 1.0)
         self.history_scale.set_value(5)
         self.history_scale.connect("value-changed", self.cb_history_scale)
+        self.history_scale.connect("change-value", self.cb_history_scale2)
         history_scale_hbox.pack_start(self.history_scale, True, True, 0)
         self.history_scale.show()
         frame.add(history_scale_hbox)
@@ -1305,7 +1309,7 @@ class mixprefs:
 
         # ReplayGain controls
 
-        frame = Gtk.Frame.new(" %s " % _('Player Loudness Normalisation'))
+        frame = Gtk.Frame.new(" %s " % _('Player Loudness Normalization'))
         frame.set_border_width(3)
         outervbox.pack_start(frame, False, False, 0)
         vbox = Gtk.VBox()
@@ -1672,7 +1676,7 @@ class mixprefs:
         frame.set_border_width(3)
         box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         frame.add(box)
-        port_grab_checkbutton = Gtk.CheckButton.new_with_label(_("Show the VoIP application port capture button in the main application window."))
+        port_grab_checkbutton = Gtk.CheckButton.new_with_label(_("Show the VoIP application port capture button in the main application window"))
         port_grab_checkbutton.connect("toggled", lambda w: parent.port_grab_button.set_visible(w.get_active()))
         port_grab_checkbutton.set_border_width(3)
         port_grab_checkbutton.set_margin_start(6)
@@ -1735,7 +1739,7 @@ class mixprefs:
         lw = licence_window.LicenceWindow()
         lw.set_border_width(1)
         lw.set_shadow_type(Gtk.ShadowType.ETCHED_IN)
-        label = Gtk.Label.new(_('Licence'))
+        label = Gtk.Label.new(_('License'))
         nb.append_page(lw, label)
         lw.show()
         label.show()
