@@ -15,8 +15,6 @@
 #   along with this program in the file entitled COPYING.
 #   If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import print_function
-
 __all__ = ["ArgumentParserImplementation", "ProfileManager"]
 
 import os
@@ -32,6 +30,9 @@ import glob
 import uuid
 import datetime
 import subprocess
+import gettext
+import configparser
+import socket
 from functools import partial
 from collections import defaultdict
 
@@ -39,7 +40,7 @@ import dbus
 import dbus.service
 from dbus.mainloop.glib import DBusGMainLoop
 DBusGMainLoop(set_as_default=True)
-import glib
+from gi.repository import GLib as glib
 
 from idjc import FGlobs
 from idjc import PGlobs
@@ -47,20 +48,18 @@ from ..utils import Singleton
 from ..utils import PathStr
 
 
-import gettext
 t = gettext.translation(FGlobs.package_name, FGlobs.localedir, fallback=True)
 _ = t.gettext
 
 
-
 # The name of the default profile.
 default = "default"
 
 
 # Regular expressions of files to copy when cloning a profile.
 config_files = ("config", "controls", "left_session", "main_session",
-    "main_session_files_played", "playerdefaults",
-    "right_session", "interlude_session", "effects[12]_session", "s_data",
+    "main_session_files_played", "playerdefaults", "s_data",
+    "right_session", "interlude_session", "miniplayers_p[1234]b[123]_session",
     "ports-.+-.+")
 
 
@@ -68,25 +67,19 @@ class ArgumentParserError(Exception):
     pass
 
 
-
 class ArgumentParser(argparse.ArgumentParser):
     def error(self, text):
         raise ArgumentParserError(text)
 
-
     def exit_with_message(self, text):
         """This is just error on the superclass."""
 
         super(ArgumentParser, self).error(text)
 
 
-
-class ArgumentParserImplementation(object):
+class ArgumentParserImplementation(metaclass=Singleton):
     """To parse the command line arguments, if any."""
 
-    __metaclass__ = Singleton
-
-
     def __init__(self, args=None, description=None, epilog=None):
         if args is None:
             args = sys.argv[1:]
@@ -96,8 +89,8 @@ class ArgumentParserImplementation(object):
         if description is None:
             description = PGlobs.app_longform
 
-        ap = self._ap = ArgumentParser(description=description, epilog=epilog,
-                                                                add_help=False)
+        ap = self._ap = ArgumentParser(description=description,
+                                       epilog=epilog, add_help=False)
         ap.add_argument("-h", "--help", action="help", help=_('show this help '
         'message and exit -- additional help is available on each of the '
         'sub-commands for example: "%(prog)s run --help" shows the help '
@@ -106,7 +99,7 @@ class ArgumentParserImplementation(object):
                 version=FGlobs.package_name + " " + FGlobs.package_version,
                 # TC: a command line option help string.
                 help=_("show the version number and exit"))
-        sp = self._sp = ap.add_subparsers()
+        sp = self._sp = ap.add_subparsers(required=True)
         # TC: a command line option help string.
 
         sp_run = sp.add_parser("run", add_help=False, help=_("run the main "
@@ -151,6 +144,10 @@ class ArgumentParserImplementation(object):
 
         sp_run.add_argument("-h", "--help", action="help",
                                     help=_('show this help message and exit'))
+        if os.getenv("idjcdev"):
+            sp_run.add_argument("-T", "--testing", dest="testing",
+                    action="store_true",
+                    help=_("developer mode for backend testing"))
         sp_run.add_argument("-d", "--dialog", dest="dialog", nargs=1,
                 choices=("true", "false"),
                 help=_("""force the appearance or non-appearance of the
@@ -246,55 +243,46 @@ class ArgumentParserImplementation(object):
                 help=_('show this help message and exit'))
         sp_ls.add_argument("--dummyarg", dest="ls", help=argparse.SUPPRESS)
 
-
     def parse_args(self):
         try:
             return self._ap.parse_args(self._args)
-        except ArgumentParserError as e:
+        except (ArgumentParserError, TypeError) as e:
             try:
-                for cmd in self._sp.choices.iterkeys():
+                for cmd in self._sp.choices.keys():
                     if cmd in self._args:
-                        raise
+                        raise ArgumentParserError(e) from e
                 return self._ap.parse_args(self._args + ["run"])
             except ArgumentParserError:
                 self._ap.exit_with_message(str(e))
 
-
     def error(self, text):
         self._ap.exit_with_message(text)
 
-
     def exit(self, status=0, message=None):
         self._ap.exit(status, message)
 
 
-
 class DBusUptimeReporter(dbus.service.Object):
     """Supply uptime to other idjc instances."""
 
-
     interface_name = PGlobs.dbus_bus_basename + ".profile"
     obj_path  = PGlobs.dbus_objects_basename + "/uptime"
 
-
     def __init__(self):
         self._uptime_cache = defaultdict(float)
         self._interface_cache = {}
         # Defer base class initialisation.
 
-
     @dbus.service.method(interface_name, out_signature="d")
     def get_uptime(self):
         """Broadcast uptime from the current profile."""
 
         return self._get_uptime()
 
-
     def activate_for_profile(self, bus_name, get_uptime):
         self._get_uptime = get_uptime
         dbus.service.Object.__init__(self, bus_name, self.obj_path)
 
-
     def get_uptime_for_profile(self, profile):
         """Ask and return the uptime of an active profile.
 
@@ -306,11 +294,9 @@ class DBusUptimeReporter(dbus.service.Object):
         Supports synchronous mode in the absence of an event loop.
         """
 
-
         def rh(retval):
             self._uptime_cache[profile] = retval
 
-
         def eh(exception):
             try:
                 del self._uptime_cache[profile]
@@ -321,7 +307,6 @@ class DBusUptimeReporter(dbus.service.Object):
             except KeyError:
                 pass
 
-
         try:
             interface = self._interface_cache[profile]
         except KeyError:
@@ -344,13 +329,11 @@ class DBusUptimeReporter(dbus.service.Object):
             return interface.get_uptime()
 
 
-
 # Profile length limited for practical reasons. For more descriptive
 # purposes the nickname parameter was created.
 MAX_PROFILE_LENGTH = 18
 
 
-
 def profile_name_valid(p):
     try:
         dbus.validate_bus_name("com." + p)
@@ -360,7 +343,6 @@ def profile_name_valid(p):
     return len(p) <= MAX_PROFILE_LENGTH
 
 
-
 class ProfileError(Exception):
     """General purpose exception used within the ProfileManager class.
 
@@ -372,11 +354,9 @@ class ProfileError(Exception):
         self.gui_text = str2
 
 
-
 def profileclosure(cmd, name):
     """A factory function of sorts."""
 
-
     busbase = PGlobs.dbus_bus_basename
     def inner(profname):
         return cmd(".".join((busbase, profname)))
@@ -384,24 +364,19 @@ def profileclosure(cmd, name):
     return staticmethod(inner)
 
 
-
-class ProfileManager(object):
+class ProfileManager(metaclass=Singleton):
     """The profile gives each application instance a unique identity.
 
     This identity extends to the config file directory if present,
     to the JACK application ID, to the DBus bus name.
     """
 
-    __metaclass__ = Singleton
-
-
     _profile = _dbus_bus_name = _profile_dialog = _init_time = None
     _iconpathname = PGlobs.default_icon
 
     _textoptionals = ("nickname", "description")
     _optionals = ("icon",) + _textoptionals
 
-
     def __init__(self):
         ap = ArgumentParserImplementation()
         args = ap.parse_args()
@@ -448,6 +423,10 @@ class ProfileManager(object):
 
         self._session_type, self._session_dir, self._session_name, \
                             self._session_uuid = self._parse_session(ap, args)
+        try:
+            self._testing = args.testing
+        except AttributeError:
+            self._testing = False
 
         if self._session_dir is None:
             # Not in session mode so do the profile init stuff.
@@ -472,7 +451,7 @@ class ProfileManager(object):
                 if not profile_name_valid(profile):
                     ap.error(_('profile name is bad'))
 
-                if profile not in os.walk(PGlobs.profile_dir).next()[1]:
+                if profile not in next(os.walk(PGlobs.profile_dir))[1]:
                     ap.error(_('profile %s does not exist') % profile)
 
                 if self._profile_has_owner(profile):
@@ -508,7 +487,7 @@ class ProfileManager(object):
                                                                 verbose=True)
             if self._profile is None:
                 ap.error(_("no profile is set"))
-                
+
         else:
             claim = "session." + self._session_name
             try:
@@ -517,23 +496,78 @@ class ProfileManager(object):
                 ap.error(_("failed to grab bus name -- "
                     "another session by the same name appears to be running"))
 
+        config = configparser.RawConfigParser()
+        config.read(self.basedir / 'config')
+        try:
+            PGlobs.num_micpairs = config.getint(
+                                        'resource_count', 'num_micpairs') // 2
+        except configparser.Error:
+            pass
+        try:
+            count = config.getint('resource_count', 'num_streamers')
+        except configparser.Error:
+            pass
+        else:
+            PGlobs.num_streamers = count
+            PGlobs.num_encoders = count
+        try:
+            PGlobs.num_recorders = config.getint(
+                                        'resource_count', 'num_recorders')
+        except configparser.Error:
+            pass
 
+        if self.session_uuid is None:
+            if args.jackserver is None:
+                os.environ["jack_parameter"] = "default"
+            else:
+                os.environ["jack_parameter"] = args.jackserver[0]
+        else:
+            os.environ["jack_parameter"] = self.session_uuid
+
+        if self.profile is not None:
+            client_id = f"{PGlobs.jack_prefix}_{self.profile}"
+        else:
+            # Client ID is by session type and name.
+            client_id = f"{PGlobs.jack_prefix}" \
+                        f"_{self.session_type}_{self.session_name}"
+
+        os.environ["app_name"] = f"{PGlobs.app_longform} " \
+                                 f"({FGlobs.package_name}) " \
+                                 f"{FGlobs.package_version}"
+        os.environ["client_id"] = client_id
+        os.environ["mic_qty"] = str(PGlobs.num_micpairs * 2)
+        os.environ["num_streamers"] = str(PGlobs.num_streamers)
+        os.environ["num_encoders"] = str(PGlobs.num_encoders)
+        os.environ["num_recorders"] = str(PGlobs.num_recorders)
+        os.environ["num_effects"] = str(PGlobs.num_mini_players)
+        os.environ["has_head"] = "1"
+        os.environ["extant_port_check"] = "1"
+        os.environ["libmp3lame_filename"] = FGlobs.libmp3lame_filename
+        os.environ["libmpg123_filename"] = FGlobs.libmpg123_filename
+        # For IPC.
+        os.environ["ui2be"] = self.basedir / "ui2be"
+        os.environ["be2ui"] = self.basedir / "be2ui"
+
+        print("jack client ID:", client_id)
+
+        socket.setdefaulttimeout(15)
+
+    @property
+    def testing(self):
+        return self._testing
 
     @property
     def profile(self):
         return self._profile
 
-
     @property
     def iconpathname(self):
         return self._iconpathname
 
-
     @property
     def dbus_bus_name(self):
         return self._dbus_bus_name
 
-
     @property
     def basedir(self):
         """The root save directory."""
@@ -543,27 +577,24 @@ class ProfileManager(object):
         else:
             return PGlobs.profile_dir / self.profile
 
-
     @property
     def session_type(self):
         """Session mode: L0 for none, L1 for Ladish L1 mode."""
 
         return self._session_type
 
-
     @property
     def session_name(self):
         """The name of the session."""
-        
+
         return self._session_name
-        
-        
+
+
     @property
     def session_uuid(self):
         """When session is JACK this will be set to something."""
-        
-        return self._session_uuid
 
+        return self._session_uuid
 
     @property
     def ports_pathname(self):
@@ -572,7 +603,6 @@ class ProfileManager(object):
         return self.basedir / ("ports-%s-%s" % (
                                         self.session_type, self.session_name))
 
-
     @property
     def title_extra(self):
         """Window title text indicating which profile is in use."""
@@ -590,7 +620,6 @@ class ProfileManager(object):
             return "  (%s)" % _('session={type}:{name}').format(
                                 type=self.session_type, name=self.session_name)
 
-
     @property
     def autoloadprofilename(self):
         """Which profile would automatically load if given the chance?"""
@@ -600,30 +629,26 @@ class ProfileManager(object):
             return None
 
         try:
-            profiledirs = os.walk(PGlobs.profile_dir).next()[1]
+            profiledirs = next(os.walk(PGlobs.profile_dir))[1]
         except (EnvironmentError, StopIteration):
             return None
 
         return al_profile if al_profile in profiledirs else None
 
-
     @property
     def profile_dialog(self):
         return self._profile_dialog
 
-
     def get_uptime(self):
         if self._init_time is not None:
             return time.time() - self._init_time
         else:
             return 0.0
 
-
     @staticmethod
     def _parse_session(ap, args):
         """User supplied session details are parsed and checked for validity."""
 
-
         def profile_check():
             if not profile_name_valid(args.profile[0]):
                 ap.error(
@@ -662,7 +687,8 @@ class ProfileManager(object):
             session_type = supported_sessions[session_type.lower()]
         except KeyError:
             ap.error(_("unknown session type: %s: must be one of %s") %
-                            (session_type, str(supported_sessions.values())))
+                     (session_type,
+                      str(list(supported_sessions.values()))))
 
         # The backend when started needs to know what session type we are using.
         os.environ["session_type"] = session_type
@@ -672,10 +698,10 @@ class ProfileManager(object):
 
         if session_dir is not None:
             session_dir = os.path.realpath(os.path.expanduser(session_dir))
-            
+
             if not os.path.isdir(session_dir):
                 ap.error(_('directory does not exist: %s') % session_dir)
-            
+
             # Use a subdir for the actual save path based on the mode and name.
             session_dir = os.path.join(session_dir, "idjc-%s-%s" % (
                                                 session_type, session_name))
@@ -691,13 +717,13 @@ class ProfileManager(object):
                     else:
                         # Perform copy of profile data.
                         try:
-                            shutil.copytree(PGlobs.profile_dir / 
+                            shutil.copytree(PGlobs.profile_dir /
                                                 args.profile[0], session_dir)
                         except EnvironmentError as e:
                             if e.errno != 17:
                                 ap.error("failed to copy data from the"
                                                 " profile directory: %s" % e)
-                    
+
             elif session_type != "JACK":
                 # Just make the empty session directory.
                 try:
@@ -710,7 +736,7 @@ class ProfileManager(object):
         if session_type == "JACK":
             if session_dir is None and args.profile is not None:
                 profile_check()
-                session_dir = PGlobs.profile_dir / args.profile[0] 
+                session_dir = PGlobs.profile_dir / args.profile[0]
             try:
                 session_uuid = uuid.UUID(args.jackserver[0])
             except TypeError:
@@ -725,7 +751,6 @@ class ProfileManager(object):
         return session_type, PathStr(session_dir), session_name, \
                             None if session_uuid is None else str(session_uuid)
 
-
     def _autoloadprofilename(self):
         """Just the file contents without checking."""
 
@@ -737,7 +762,6 @@ class ProfileManager(object):
             return None
         return al_profile
 
-
     def _auto(self, dialog, profile):
         if dialog is None and profile != default and not \
                                     os.path.isdir(PGlobs.profile_dir / profile):
@@ -747,7 +771,7 @@ class ProfileManager(object):
             if not os.path.exists(PGlobs.autoload_profile_pathname):
                 with open(PGlobs.autoload_profile_pathname, "w"):
                     pass
-            
+
             with open(PGlobs.autoload_profile_pathname, "r+") as f:
                 fcntl.flock(f, fcntl.LOCK_EX)
                 al_profile = f.readline().strip()
@@ -759,7 +783,6 @@ class ProfileManager(object):
             if dialog is None:
                 raise ProfileError(str(e), None)
 
-
     def _noauto(self):
         try:
             with open(PGlobs.autoload_profile_pathname, "r+") as f:
@@ -768,11 +791,13 @@ class ProfileManager(object):
         except IOError:
             pass
 
-
     def _cb_edit_profile(self, dialog, newprofile, oldprofile, *opts):
         busses = []
 
         try:
+            if not newprofile:
+                raise ProfileError(None, "Profile name field is blank.")
+
             try:
                 busses.append(self._grab_bus_name_for_profile(oldprofile))
                 if newprofile != oldprofile:
@@ -809,7 +834,6 @@ class ProfileManager(object):
         else:
             dialog.destroy_new_profile_dialog()
 
-
     def _delete_profile(self, dialog, profiles):
         if isinstance(profiles, str):
             profiles = [profiles]
@@ -840,7 +864,6 @@ class ProfileManager(object):
             if profile == default:
                 self._generate_default_profile()
 
-
     def _choose_profile(self, dialog, profile, verbose=False):
         if dialog.profile is None:
             try:
@@ -865,7 +888,6 @@ class ProfileManager(object):
             subprocess.Popen([FGlobs.bindir / FGlobs.package_name,
                 "run", "-p", profile], close_fds=True)
 
-
     def _generate_profile(self, newprofile, template=None, **kwds):
         if PGlobs.profile_dir is not None:
             if len(newprofile) > MAX_PROFILE_LENGTH:
@@ -933,7 +955,6 @@ class ProfileManager(object):
                                         _("could not write file %s") + fname,
                                         _("Could not write file %s.") % fname)
 
-
                 dest = PGlobs.profile_dir / newprofile
                 try:
                     shutil.copytree(tmp, dest)
@@ -954,21 +975,19 @@ class ProfileManager(object):
                 except EnvironmentError:
                     pass
 
-
     def _generate_default_profile(self):
         self._generate_profile(default, description=_("The default profile"))
 
-
     def _profile_data(self):
         a = self._autoloadprofilename()
         d = PGlobs.profile_dir
         try:
-            profdirs = os.walk(d).next()[1]
+            profdirs = next(os.walk(d))[1]
         except (EnvironmentError, StopIteration):
             return
         for profname in profdirs:
             if profile_name_valid(profname):
-                files = os.walk(d / profname).next()[2]
+                files = next(os.walk(d / profname))[2]
                 rslt = {"profile": profname}
                 for each in self._optionals:
                     try:
@@ -983,7 +1002,6 @@ class ProfileManager(object):
                 rslt["auto"] = (1 if a == profname else 0)
                 yield rslt
 
-
     def _ls(self):
         table = []
         for pd in self._profile_data():
@@ -1000,13 +1018,11 @@ class ProfileManager(object):
             print("{1} {0:{5}} {2:>16} {3} {4}".format(*(tuple(row) +
                   (MAX_PROFILE_LENGTH,))))
 
-
     _profile_has_owner = profileclosure(dbus.SessionBus().name_has_owner,
                             "_profile_has_owner")
 
-    _grab_bus_name_for_profile = profileclosure(partial(
-        dbus.service.BusName, do_not_queue=True), "_grab_bus_name_for_profile")
-
+    _grab_bus_name_for_profile = profileclosure(partial(dbus.service.BusName,
+        do_not_queue=True, bus=dbus.SessionBus()), "_grab_bus_name_for_profile")
 
     @staticmethod
     def _grab_profile_filetext(profile, filename):
@@ -1016,7 +1032,6 @@ class ProfileManager(object):
         except EnvironmentError:
             return None
 
-
     def _get_profile_dialog(self):
         from .profiledialog import ProfileDialog
 
