@@ -65,6 +65,7 @@ __all__ = ['MediaPane', 'have_songdb']
 
 AMPACHE = "Ampache"
 AMPACHE_3_7 = "Ampache 3.7"
+AMPACHE_7_0 = "Ampache 7.0"
 PROKYON_3 = "Prokyon 3"
 FUZZY, CLEAN, WHERE, DIRTY = range(4)
 
@@ -144,7 +145,7 @@ class DBAccessor(threading.Thread):
         self.password = password
         self.database = database
         self.notify = notify
-        self._handle = None  # No connections made until there is a query.
+        self._sql_connection = None  # No connections made until there is a query.
         self._cursor = None
         # Always enable setting charset to utf8
         self.extra_args = {"charset": "utf8"}
@@ -169,7 +170,7 @@ class DBAccessor(threading.Thread):
         self.jobs.append((sql_query, handler, failhandler))
         self.semaphore.release()
 
-    def close(self):
+    def worker_exit(self):
         """Clean up the worker thread prior to disposal."""
 
         if self.is_alive():
@@ -221,7 +222,7 @@ class DBAccessor(threading.Thread):
                                     pass
 
                                 try:
-                                    self._handle.close()
+                                    self._sql_connection.close()
                                 except Exception:
                                     pass
 
@@ -231,12 +232,12 @@ class DBAccessor(threading.Thread):
                             notify(_('Connecting'))
                             trycount += 1
                             try:
-                                self._handle = sql.Connection(
+                                self._sql_connection = sql.Connection(
                                     host=self.hostname, port=self.port,
                                     user=self.user, passwd=self.password,
                                     db=self.database, connect_timeout=6,
                                     **self.extra_args)
-                                self._cursor = self._handle.cursor()
+                                self._cursor = self._sql_connection.cursor()
                             except sql.Error as e:
                                 notify(_("Connection failed (try %d)") %
                                                                     trycount)
@@ -247,7 +248,7 @@ class DBAccessor(threading.Thread):
                                 # processes try to access the database,
                                 # so set autocommit to 1
                                 try:
-                                    self._handle.autocommit(True)
+                                    self._sql_connection.autocommit(True)
                                 except sql.MySQLError:
                                     notify(_('Connected: autocommit mode failed'))
                                 else:
@@ -267,7 +268,7 @@ class DBAccessor(threading.Thread):
             except Exception:
                 pass
             try:
-                self._handle.close()
+                self._sql_connection.close()
             except Exception:
                 pass
             notify(_('Disconnected'))
@@ -281,7 +282,7 @@ class DBAccessor(threading.Thread):
     @thread_only
     def disconnect(self):
         try:
-            self._handle.close()
+            self._sql_connection.close()
         except sql.Error:
             idle_add(self.notify, _('Problem dropping connection'))
         else:
@@ -292,7 +293,7 @@ class DBAccessor(threading.Thread):
         """Handler may break off the cursor to pass along its data."""
 
         assert cursor is self._cursor
-        self._cursor = self._handle.cursor()
+        self._cursor = self._sql_connection.cursor()
 
 
 class UseSettings(dict):
@@ -719,9 +720,9 @@ class ViewerCommon(PageCommon):
         bitrate = model.get_value(iter, cell)
         if bitrate == 0:
             renderer.props.text = ""
-        elif self._db_type == "P3":
+        elif self._db_type == PROKYON_3:
             renderer.props.text = "{}k".format(bitrate)
-        elif bitrate > 9999 and self._db_type in (AMPACHE, AMPACHE_3_7):
+        elif bitrate > 9999 and self._db_type in (AMPACHE, AMPACHE_3_7, AMPACHE_7_0):
             renderer.props.text = "{}k".format(bitrate // 1000)
         renderer.props.xalign = 1.0
 
@@ -1035,12 +1036,13 @@ class TreePage(ViewerCommon):
                     LEFT JOIN albums on tracks.album = albums.name
                      AND tracks.artist = albums.artist
                     ORDER BY tracks.artist, album, tracknumber, title"""
-        elif self._db_type in (AMPACHE, AMPACHE_3_7):
-            query = """SELECT
+        elif self._db_type in (AMPACHE, AMPACHE_3_7, AMPACHE_7_0):
+            _count = "" if self._db_type in (AMPACHE, AMPACHE_3_7) else "_count"
+            query = f"""SELECT
                     album.name as album,
                     IFNULL(album.prefix, "") as alb_prefix,
                     album.year as year,
-                    album.disk as disk,
+                    album.disk{_count} as disk,
                     song.album as album_id,
                     IFNULL(track, 0) as tracknumber,
                     title,
@@ -1436,6 +1438,11 @@ class FlatPage(ViewerCommon):
                     """)}
     }
     _queries_table[AMPACHE_3_7] = _queries_table[AMPACHE]
+    _queries_table[AMPACHE_7_0] = {}
+    _queries_table[AMPACHE_7_0][FUZZY] = (CLEAN, _queries_table[AMPACHE][FUZZY][1].replace(
+                                          "album.disk", "album.disk_count"))
+    _queries_table[AMPACHE_7_0][WHERE] = (DIRTY, _queries_table[AMPACHE][WHERE][1].replace(
+                                          "album.disk", "album.disk_count"))
 
     def _cb_update(self, widget):
         self._old_cat_data = self.catalogs.copy_data()
@@ -1710,7 +1717,7 @@ class CatalogsPage(PageCommon):
 
     def activate(self, *args, **kwargs):
         PageCommon.activate(self, *args, **kwargs)
-        self.tree_view.get_column(0).set_visible(self._db_type in (AMPACHE, AMPACHE_3_7))
+        self.tree_view.get_column(0).set_visible(self._db_type in (AMPACHE, AMPACHE_3_7, AMPACHE_7_0))
         self.refresh.clicked()
 
     def deactivate(self, *args, **kwargs):
@@ -1753,7 +1760,7 @@ class CatalogsPage(PageCommon):
             self.interface.update(self.list_store)
 
     def _on_refresh(self, widget):
-        if self._db_type in (AMPACHE, AMPACHE_3_7):
+        if self._db_type in (AMPACHE, AMPACHE_3_7, AMPACHE_7_0):
             self.refresh.set_sensitive(False)
             self.tree_view.set_model(None)
             if self._db_type == AMPACHE:
@@ -1914,7 +1921,7 @@ class MediaPane(Gtk.VBox):
         else:
             try:
                 for i in range(1, 4):
-                    getattr(self, "_acc{}".format(i)).close()
+                    getattr(self, "_acc{}".format(i)).worker_exit()
             except AttributeError:
                 pass
             else:
@@ -2037,8 +2044,21 @@ class MediaPane(Gtk.VBox):
 
     def _stage_11(self, acc, request, cursor, notify, rows):
         if self.schema_test("path", cursor.fetchall()):
-            notify('Found Ampache 3.7+ schema')
-            self._hand_over(AMPACHE_3_7)
+            request(("DESCRIBE album",), self._stage_12, self._fail_2)
         else:
             notify('Unrecognised database')
             self._safe_disconnect()
+
+    def _stage_12(self, acc, request, cursor, notify, rows):
+        rslt = cursor.fetchall()
+        if self.schema_test("disk_count", rslt):
+            notify('Found Ampache 7.0+ schema')
+            self._hand_over(AMPACHE_7_0)
+            return
+        if self.schema_test("disk", rslt):
+            notify('Found Ampache 3.7+ pre 7.0 schema')
+            self._hand_over(AMPACHE_3_7)
+            return
+
+        notify('Unrecognised database :(')
+        self._safe_disconnect()
